/*
 * (C) Copyright 2002
 * Gary Jennejohn, DENX Software Engineering, <gj@denx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <common.h>

#include <arm940t.h>

/*#include <asm/arch/symbol.h>
#include <asm/arch/porting.h>
#include <asm/arch/serial.h>
#include <asm/arch/star_uart.h>*/
#include <asm/arch/hardware.h>
/******************************************************************************
 *
 * FUNCTION:  Uart_Configure_Baud_Rate
 * PURPOSE:   
 *
 ******************************************************************************/
void Uart_Configure_Baud_Rate(unsigned int baud_rate, unsigned int input_clock)
{
    unsigned int  baud_rate_divisor;
    unsigned int  updated_input_clock;

    /*
     * For 48MHz input clock and new UART Pre_Scalar design
     */
    input_clock = 48000000;
    updated_input_clock = (input_clock/325) * 100;
    	    	
    baud_rate_divisor = updated_input_clock/(16 * baud_rate);    	

    
    if (baud_rate_divisor)
    {    
        UART_WRITE_DLM((baud_rate_divisor & 0x0000FF00) >> 8);        
        UART_WRITE_DLL((baud_rate_divisor & 0x000000FF));
    }
}


/******************************************************************************
 *
 * FUNCTION:  Uart_Control_Line_Control
 * PURPOSE:   
 *
 ******************************************************************************/
void Uart_Configure_Line_Control(u_int32 baud_rate, u_int32 word_len, u_int32 parity, u_int32 stop_bit)
{
    u_int32  uart_LCR = 0;
    
    
    // configure baud rate setting
    Uart_Configure_Baud_Rate(baud_rate, UART_INPUT_CLOCK);
    
    // configure word length setting
    if (word_len == WORD_FIVE_BITS)
    {
        uart_LCR |= WORD_LENGTH_5;
    }
    else if (word_len == WORD_SIX_BITS)
    {
        uart_LCR |= WORD_LENGTH_6;
    }
    else if (word_len == WORD_SEVEN_BITS)
    {
        uart_LCR |= WORD_LENGTH_7;
    }
    else if (word_len == WORD_EIGHT_BITS)
    {
        uart_LCR |= WORD_LENGTH_8;
    }
    
    // configure parity setting
    if (parity == NONE_PARITY)
    {
        uart_LCR |= PARITY_CHECK_NONE;
    }
    else if (parity == EVEN_PARITY)
    {
        uart_LCR |= PARITY_CHECK_EVEN;
    }
    else if (parity == ODD_PARITY)
    {
        uart_LCR |= PARITY_CHECK_ODD;
    }
    else if (parity == ONE_PARITY)
    {
        uart_LCR |= PARITY_CHECK_STICK_ONE;
    }
    else if (parity == ZERO_PARITY)
    {
        uart_LCR |= PARITY_CHECK_STICK_ZERO;
    }

    // configure stop bit setting
    if (stop_bit == ONE_STOP_BIT)
    {
        uart_LCR |= STOP_BIT_1;
    }
    else if ((stop_bit == ONE_HALF_STOP_BIT) || (stop_bit == TWO_STOP_BIT))
    {
        uart_LCR |= STOP_BIT_2;
    }
    
    UART_LCR = uart_LCR;
}


/******************************************************************************
 *
 * FUNCTION:  Uart_Configure_FIFO
 * PURPOSE:   
 *
 ******************************************************************************/
void Uart_Configure_FIFO(u_int32 fifo_flag, u_int32 rx_fifo_trigger_level)
{
    u_int32  uart_FCR = 0;

    if (fifo_flag == TX_RX_FIFO_DISABLE)
    {
        uart_FCR = 0;
    }
    else
    {
        uart_FCR |= FIFO_ENABLE;
    	
        if (rx_fifo_trigger_level == 1)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_1;
        }
        else if (rx_fifo_trigger_level == 4)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_4;
        }
        else if (rx_fifo_trigger_level == 8)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_8;
        }
        else if (rx_fifo_trigger_level == 14)
        {
            uart_FCR |= RX_FIFO_TRIGGER_LEVEL_14;
        }
        
        // reset TX/RX FIFO
        uart_FCR |= (RX_FIFO_RESET | TX_FIFO_RESET);
    }
    
    UART_FCR = uart_FCR;
}



void serial_setbrg(void)
{

}


/*
 * Initialise the serial port with the given baudrate. The settings
 * are always 8 data bits, no parity, 1 stop bit, no start bits.
 *
 */
int serial_init(void)
{
   DECLARE_GLOBAL_DATA_PTR;
   /*
     * for 48MHz UART clock source
     */	
    UART_WRITE_PSR(2);
      
    Uart_Configure_Line_Control(gd->baudrate, WORD_EIGHT_BITS, NONE_PARITY, ONE_STOP_BIT);
        
    Uart_Configure_FIFO(TX_RX_FIFO_ENABLE, 8);    
    
    return 0;
}


void CheckRxStatus()
{
    /*
     * wait until Rx ready
     */
    do
    {
    } while (!UART_CHECK_RX_DATA_READY());
}

void CheckTxStatus()
{
    /*
     * wait until Tx ready
     */
    do
    {
    } while (!UART_CHECK_TX_FIFO_EMPTY());   
}


/*
 * Output a single byte to the serial port.
 */
void serial_putc(const char c)
{
   if (c != '\0')
    {        
        CheckTxStatus();
        
        UART_THR = (u_int32)(c & 0xFF);        
    }
    
    if (c == '\n')
    {
        CheckTxStatus();
        
        UART_THR = (u_int32)('\r' & 0xFF);        
    }      
}		

void serial_puts (const char *s)
{
	while (*s) {
		serial_putc (*s++);
	}
}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise. When the function is succesfull, the character read is
 * written into its argument c.
 */
volatile int    uart_download_start = 0;
static int      count = 0;

int serial_getc(void)
{
     char  rx_data;
    
    
    count = 0;
     
    while (!UART_CHECK_RX_DATA_READY())
    {
        /*
         * add this to prevent starvation
         */
        if (uart_download_start > 10)
        {
            count++;
            
            if (count > 100000)
                break;
        }        
    }
    
    rx_data = (char)(UART_RBR & 0xFF);
    
    return  rx_data;                
     
}

/*
 * Test whether a character is in the RX buffer
 */
int serial_tstc(void)
{
    return (UART_CHECK_RX_DATA_READY());
}

