/***************************************************************************
* Name:star_serial.c                                                       *
* Description: serial library routine                                      *
****************************************************************************/

/*#include "str9100/sys_typedef.h"
#include "str9100/except.h"
#include "str9100/star_uart.h"
#include "str9100/symbol.h"*/
#include <asm/arch/hardware.h>

#include <common.h>
/*#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>*/


#define BACKSP_KEY	0x08
#define RETURN_KEY	0x0D
#define DELETE_KEY	0x7F
#define BELL       	0x07



void UART_Init(unsigned int baudRate)
{
    /*
     * for 48MHz UART clock source
     */	
    UART_WRITE_PSR(2);
      
    Uart_Configure_Line_Control(baudRate, WORD_EIGHT_BITS, NONE_PARITY, ONE_STOP_BIT);
        
    Uart_Configure_FIFO(TX_RX_FIFO_ENABLE, 8);    
}


//#define Read_CPSR(reg,val)  asm volatile ("stt $f"#reg",%0" : "=m"(val));

UINT32 Read_CPSR()
{   	 
#if 1
    UINT32 temp;
    __asm__ __volatile__("mrs %0, cpsr"
			 : "=r" (temp)
			 :
			 : "memory");

    return temp;     
#endif    
}


void Write_CPSR(UINT32 tmp)
{
#if 1
    __asm__ __volatile__("msr cpsr, %0"
                         : 
                         :"r" (tmp)
			);
#endif    
}


void CheckRxStatus()
{
    /*
     * wait until Rx ready
     */
    do
    {
    } while (!UART_CHECK_RX_DATA_READY());
}


void CheckTxStatus()
{
    /*
     * wait until Tx ready
     */
    do
    {
    } while (!UART_CHECK_TX_FIFO_EMPTY());   
}


char sys_getch(void)
{
    char  rx_data;
     
    if (UART_CHECK_RX_DATA_READY())
    {
        rx_data = (char)(UART_RBR & 0xFF);
    }
    else
    {
        rx_data = 0;
    }	
    
    return  rx_data;
}				


char sys_getchar(void)
{
    char  rx_data;
     
    while (!UART_CHECK_RX_DATA_READY())
    {
        // wait..........
    }
     
    rx_data = (char)(UART_RBR & 0xFF);
     
    //printf("<%02x>",rx_data);     
     
    return  rx_data;
}				


volatile int    uart_download_start = 0;
static int      count = 0;

char uart_getchar(void)
{   
    char  rx_data;
    
    
    count = 0;
     
    while (!UART_CHECK_RX_DATA_READY())
    {
        /*
         * add this to prevent starvation
         */
        if (uart_download_start > 10)
        {
            count++;
            
            if (count > 100000)
                break;
        }        
    }
    
    rx_data = (char)(UART_RBR & 0xFF);
    
    return  rx_data;                 
}				


void sys_putchar(char Ch)
{
    if (Ch != '\0')
    {        
        CheckTxStatus();
        
        UART_THR = (u_int32)(Ch & 0xFF);        
    }
    
    if (Ch == '\n')
    {
        CheckTxStatus();
        
        UART_THR = (u_int32)('\r' & 0xFF);        
    }      
}


void sys_putc(char Ch)
{
    CheckTxStatus();
    
    UART_THR = (u_int32)(Ch & 0xFF);        
    
    if (Ch == '\n')
    {
        CheckTxStatus();
        
        UART_THR = (u_int32)('\r' & 0xFF);        
    }      
}


void sys_putstr(char *str)
{
    char *cp;
    
    for (cp = str; *cp != 0; cp++)
    {
        sys_putchar(*cp);
    }
}


INT32 sys_printf(const char *f, ...)	/* variable arguments */
{
    INT32      cnt, i;	    
    va_list    arg_ptr;
    UINT32     mode;
    char       buffer[256];
      
    
    /* 
     * disable IRQs and FIQs 
     */
    mode = Read_CPSR();
    Write_CPSR(mode | F_BIT | I_BIT);

    /*
     * put the character to buffer
     */
    va_start(arg_ptr, f);
    cnt = vsprintf(&buffer[0], f, arg_ptr);
    va_end(arg_ptr);   
	
    /*
     * output the buffer
     */
    i = 0;
    
    while (buffer[i])	
    {
        sys_putchar(buffer[i]);
        
        i++;
    }

    /* 
     * restore the previous mode 
     */
    Write_CPSR(mode);
   
    return 0;
}


int sys_scanf(char *buf)
{
    char      *cp;
    char      data;
    UINT32    count;
    
    
    count = 0;
    cp = buf;
    
    do
    {
        data = sys_getchar();

        switch (data)
        {
            case RETURN_KEY:
                if(count < 256)
                {
                    *cp = '\0';
                    
                    sys_putchar('\n');
                }          
                break;
                
            case BACKSP_KEY:
            case DELETE_KEY:
                if(count)
                {
                    count--;
                    
                    *(--cp) = '\0';
                    
                    sys_putstr("\b \b");
                }         
                break;
                
            default:         
                if( data > 0x1F && data < 0x7F && count < 256)
                {
                    *cp = (char)data;
                    
                    cp++;
                    
                    count++;
                    
                    sys_putchar(data);
                }
                
                break;
        }
    } while (data != RETURN_KEY);  
    
    return (count);
}


// End of file - star_serial.c

