/*#include "armboot.h"
#include "str9100/porting.h"
#include "str9100/symbol.h"
#include <malloc.h>
#include "command.h"
#include "str9100/star_gsw.h"
#include "str9100gsw.h"
#include "net.h"
#include "str9100/star_gpio.h"
*/
#include <common.h>
#include <asm/arch/hardware.h> 


#ifdef CONFIG_STR9100_GSW


static orn_device	*orn_dev = (orn_device *)NULL;


static u8		orn_initialized = 0;

static char		default_mac[] = {0x00, 0xaa, 0xbb, 0xcc, 0xdd, 0x10};

static const char	version[] = "Orion GigaBit Switch Driver, (Linux Kernel 2.4) - by Star\n";


static u32		orn_icplus175c_mii_mode = 0;		/*
								 * 0 : dual MII mode
								 * 1 : single MII mode
								 */


static u32		gsw_mac_0_phy_addr;
static u32		gsw_mac_1_phy_addr;



/* wait when write ARL Entry in mini-sec */
#define ARL_CMD_WAIT		(1000)


/*
 * Debug Macro
 * The Lower ETH_DEBUG The Higher Priority of Print
 */
#if 0
#define ETH_DEBUG		0
#endif


#if defined(ETH_DEBUG) && (ETH_DEBUG >= 0)

#define DBG_PRINT(_lvl, _x...)	if((_lvl) <= ETH_DEBUG) {printf("%15s(%05d):",__FILE__,__LINE__); printf(_x);}

#define DBG_FUNC(_lvl)		DBG_PRINT((_lvl),"%s()\n",__FUNCTION__);


/*
 * ASSERT: Note!!! if (_x) test fail will return in (_err)
 */
#define ASSERT(_x)		if(!(_x)) DBG_PRINT(0, "!("#_x")\n")

#else

#define DBG_PRINT(_lvl, _x...)
#define DBG_FUNC(_lvl)
#define ASSERT(_x)

#endif /* ETH_DEBUG */


/*
 * driver interfaces
 */
extern int		eth_init(bd_t *bd);
extern void		eth_halt(void);
extern int		eth_rx(void);
extern int		eth_send(volatile void *packet, int length);


/*
 * driver utilties
 */
static int		orn_drv_config(orn_device *dev);
static int		orn_drv_alloc_ring(orn_device *dev);
static void 		*orn_drv_alloc_mem(u32 bytes, u8 clear);
static void 		orn_enqueue_tail(drv_pkt_queue *queue, drv_pkt_desc *pkt);
static void 		orn_enqueue_head(drv_pkt_queue *queue, drv_pkt_desc *pkt);
static drv_pkt_desc	*orn_dequeue(drv_pkt_queue *queue);
static drv_pkt_desc	*orn_get_free_pkt(orn_device *dev);
static int		orn_save_free_pkt(orn_device *dev, drv_pkt_desc *pkt);


static u32		orn_set_phy_addr(u32 mac_port, u32 phy_addr);
static int		orn_phy_read(u8 phy_addr, u8 phy_reg, u16 *value);
static int		orn_phy_write(u8 phy_addr, u8 phy_reg, u16 value);


static void		orn_configure_asix_gsw_cicada_phy(void);
static void		orn_configure_asix_gsw_agere_phy(void);
static void		orn_configure_agere_phy_patch(u8 phy_addr);
static void		orn_configure_icplus_phy_icplus_phy(void);
static void             orn_configure_icplus_175c_phy4(void);


/*
 * driver services
 */
static int		orn_drv_open(orn_device *dev);
static int		orn_drv_close(orn_device *dev);
static int		orn_drv_rx(orn_device *dev);
static int		orn_drv_send(orn_device *dev);
static void		orn_drv_send_complete(orn_device *dev);



static void orn_print_packet( u8 *buf, int length )
{
#if (defined(ETH_DEBUG) && (ETH_DEBUG >= 2))

        int i;
        int remainder;
        int lines;


        printf("Packet of length %d \n", length);

        lines = length / 16;
        remainder = length % 16;

        for (i = 0; i < lines ; i++)
        {
                int cur;

                for (cur = 0; cur < 8; cur++)
                {
                        u8 a, b;

                        a = *(buf++);
                        b = *(buf++);
                        
                        printf("%02x%02x ", a, b);
                }
                
                printf("\n");
        }
        
        for (i = 0; i < remainder/2; i++)
        {
                u8 a, b;

                a = *(buf++);
                b = *(buf++);
                
                printf("%02x%02x ", a, b);
        }
        
        printf("\n");
#endif		
}


// for STR9100 after ECO
/******************************************************************************
 *
 * FUNCTION:  orn_set_phy_addr
 * PURPOSE:
 *
 ******************************************************************************/
static u32 orn_set_phy_addr(u32 mac_port, u32 phy_addr)
{
	u32	status = 0;    /* for failure indication */


	if ((mac_port > 1) || (phy_addr > 31))
	{
		return status;
	}


	if (mac_port == 0)
	{
		GSW_PORT_MIRROR &= ~(0x3 << 0);    /* clear bit[1:0] for PHY_ADDR[1:0] */
        
		GSW_PORT_MIRROR &= ~(0x3 << 4);    /* clear bit[5:4] for PHY_ADDR[3:2] */
        
		GSW_QUEUE_STATUS_TEST_1 &= ~(0x1 << 25);    /* clear bit[25] for PHY_ADDR[4] */
        
        
		GSW_PORT_MIRROR |= (((phy_addr >> 0) & 0x3) << 0);
        
		GSW_PORT_MIRROR |= (((phy_addr >> 2) & 0x3) << 4);
        
		GSW_QUEUE_STATUS_TEST_1 |= (((phy_addr >> 4) & 0x1) << 25);
        
		status = 1;    /* for ok indication */
	}
	else
	{
		GSW_PORT_MIRROR &= ~(0x1 << 6);    /* clear bit[6] for PHY_ADDR[0] */
        
		GSW_PORT_MIRROR &= ~(0x7 << 8);    /* clear bit[10:8] for PHY_ADDR[3:1] */
        
		GSW_QUEUE_STATUS_TEST_1 &= ~(0x1 << 26);    /* clear bit[26] for PHY_ADDR[4] */
        
        
		GSW_PORT_MIRROR |= (((phy_addr >> 0) & 0x1) << 6);
        
		GSW_PORT_MIRROR |= (((phy_addr >> 1) & 0x7) << 8);
        
		GSW_QUEUE_STATUS_TEST_1 |= (((phy_addr >> 4) & 0x1) << 26);
        
		status = 1;    /* for ok indication */
	}
    
	return status;
}



/******************************************************************************
 *
 * FUNCTION:  orn_phy_read
 * PURPOSE:
 *
 ******************************************************************************/
static int orn_phy_read(u8 phy_addr, u8 phy_reg, u16 *value)
{
    // for STR9100 ECO
    u_int32 volatile    status;
    

    if (phy_addr > 31)
    {
        return 1;    /* for failure indication */
    }

    
    /* 
     * clear previous rw_ok status
     */
    GSW_PHY_CONTROL = (0x1 << 15);


    GSW_QUEUE_STATUS_TEST_1 &= ~(0xF << 16);    /* clear bit[19:16] for PHY_ADDR[4:1] */
    
    GSW_PHY_CONTROL &= ~(0x1 << 0);    /* clear bit[0] for PHY_ADDR[0] */


    GSW_QUEUE_STATUS_TEST_1 |= (((phy_addr >> 1) & 0xF) << 16);
    
    GSW_PHY_CONTROL = ((phy_addr & 0x1) | 
                       ((phy_reg & 0x1F) << 8) | 
                       (0x1 << 14));    


    while (1)
    {
    	status = GSW_PHY_CONTROL;
        
        if (status & (0x1 << 15))
        {
            // clear the rw_ok status, and clear other bits value
            GSW_PHY_CONTROL = (0x1 << 15);
        	
            *value = (u_int16)((status >> 16) & 0xFFFF);
            
            return 0;    /* for ok indication */
        }
    }
    
    return 1;
}



/******************************************************************************
 *
 * FUNCTION:  orn_phy_write
 * PURPOSE:
 *
 ******************************************************************************/
static int orn_phy_write(u8 phy_addr, u8 phy_reg, u16 value)
{
    // for STR9100 ECO

    if (phy_addr > 31)
    {
        return 1;    /* for failure indication */
    }


    /* 
     * clear previous rw_ok status
     */
    GSW_PHY_CONTROL = (0x1 << 15);
	/* clear bit[19:16] for PHY_ADDR[4:1] */
    GSW_QUEUE_STATUS_TEST_1 &= ~(0xF << 16);  
	 /* clear bit[0] for PHY_ADDR[0] */ 
    GSW_PHY_CONTROL &= ~(0x1 << 0);    

    GSW_QUEUE_STATUS_TEST_1 |= (((phy_addr >> 1) & 0xF) << 16);
    
    GSW_PHY_CONTROL = ((phy_addr & 0x1) | 
                       ((phy_reg & 0x1F) << 8) |
                       (0x1 << 13) |
                       ((value & 0xFFFF) << 16));


    while (1)
    {       
        if ((GSW_PHY_CONTROL) & (0x1 << 15))
        {
            // clear the rw_ok status, and clear other bits value
            GSW_PHY_CONTROL = (0x1 << 15);

            return 0;    /* for ok indication */
        }
    }

    return 1;    /* for failure indication */
}



static void orn_enqueue_tail(drv_pkt_queue *queue, drv_pkt_desc *pkt)
{
	DBG_FUNC(3);
		
	ASSERT(pkt);


	if(!pkt) return;
    
	if(queue->tail == NULL)
	{
		queue->head = queue->tail = pkt;
	}
	else
	{
    		queue->tail->next = pkt;
    		
    		queue->tail = pkt;
	}
    
	pkt->next = NULL;
}



static void orn_enqueue_head(drv_pkt_queue *queue, drv_pkt_desc *pkt)
{
	DBG_FUNC(3);
	  
	ASSERT(pkt);


	if(!pkt) return;
		
	pkt->next = queue->head;
	
	if(queue->head == NULL)
	{
		queue->head = queue->tail = pkt;
	}
	else
	{
		queue->head = pkt;
	}
}



static drv_pkt_desc *orn_dequeue(drv_pkt_queue *queue)
{
	drv_pkt_desc *pkt;


	DBG_FUNC(3);
		  
	pkt = queue->head;
		
	if(queue->head != NULL)
	{
		if(queue->head == queue->tail) 
		{
			queue->head = queue->tail = NULL;
		}
		else
		{
			queue->head = pkt->next;
		}
		
		pkt->next = NULL;
	}
		
	return pkt;
}


static int orn_drv_config(orn_device *dev)
{
	u8 i, j;
	u8 *config_mac;


	DBG_FUNC(2);
	
	ASSERT(dev);
	
	dev->num_tx_desc = NUM_TX_DESC;
	dev->num_rx_desc = NUM_RX_DESC;
	dev->num_free_pkt = NUM_FREE_DESC;


	/* configure VLAN group */
	for(i = 0; i < 8; i++)
	{
  		dev->vlan[i].gid = i;
  		dev->vlan[i].vid = DEFAULT_VLAN_ID;
  		dev->vlan[i].port_map = 0;
  		dev->vlan[i].tag_port_map = 0;
  		dev->vlan[i].mac[0] = 0;	/* 32-bit */
  		dev->vlan[i].mac[1] = 0;	/* 32-bit */
	}


  	/* configure VLAN group 0, 1, 2 port map */
  	dev->vlan[0].port_map = (GSW_VLAN_PORT_0 | GSW_VLAN_PORT_1 | GSW_VLAN_PORT_CPU);
  	
	dev->vlan[1].port_map = (GSW_VLAN_PORT_0 | GSW_VLAN_PORT_CPU);
	
	dev->vlan[2].port_map = (GSW_VLAN_PORT_1 | GSW_VLAN_PORT_CPU);
	


	/* configure VLAN group mac addr */
	for (i = 0; i < 3; i++)
	{
  		config_mac = (u8 *)dev->vlan[i].mac;
  		
	  	for (j = 0; j < 6; j++)
	  	{
			config_mac[j] = default_mac[j];
		}
	}


	/* configure MAC port 0 PVID */
	dev->port[0].pvid = dev->vlan[1].gid;
	dev->port[0].config = 0;
	dev->port[0].state = 0;

	/* configure MAC port 1 PVID */
	dev->port[1].pvid = dev->vlan[2].gid;
	dev->port[1].config = 0;
	dev->port[1].state = 0;

	/* configure CPU port PVID */
	dev->port[2].pvid = dev->vlan[0].gid;
	dev->port[2].config = 0;
	dev->port[2].state = 0;


	/* assign mac addr */
	config_mac = (u8 *)dev->mac_addr;
	
	for(i = 0; i < 6; i++)
	{
		config_mac[i] = default_mac[i];
	}
	
	return 0;
}


static int orn_drv_alloc_ring(orn_device *dev)
{
	u16 i, totoal_pkt_require = 0;
	u8 *buf;


	DBG_FUNC(2);
	
	ASSERT(dev);

  
	/* allocate descriptors */
	dev->hw_tx_desc = (orn_tx_desc *)orn_drv_alloc_mem(dev->num_tx_desc * sizeof(orn_tx_desc), TRUE);
	dev->hw_rx_desc = (orn_rx_desc *)orn_drv_alloc_mem(dev->num_rx_desc * sizeof(orn_rx_desc), TRUE);
  
	dev->drv_tx_desc = (drv_pkt_desc **)orn_drv_alloc_mem(dev->num_tx_desc * sizeof(drv_pkt_desc *), FALSE);
	dev->drv_rx_desc = (drv_pkt_desc **)orn_drv_alloc_mem(dev->num_rx_desc * sizeof(drv_pkt_desc *), FALSE);
  
	ASSERT(dev->hw_tx_desc);
	ASSERT(dev->hw_rx_desc);
	ASSERT(dev->drv_tx_desc);
	ASSERT(dev->drv_rx_desc);


	if(!(dev->hw_tx_desc && dev->hw_rx_desc && dev->drv_tx_desc && dev->drv_rx_desc))
	{
		printf("fail_1\n");
		return 0;
	}


	/* allocate pkt pool */
	totoal_pkt_require = dev->num_rx_desc + dev->num_free_pkt;
  
	dev->pkt_pool = (drv_pkt_desc *)orn_drv_alloc_mem(totoal_pkt_require * sizeof(drv_pkt_desc), FALSE);
	
	dev->buf_pool = (u8 *)orn_drv_alloc_mem(totoal_pkt_require * PKT_BUFF_SIZE, FALSE);
  
	ASSERT(dev->pkt_pool);
	ASSERT(dev->buf_pool);
  
	if(!(dev->pkt_pool && dev->buf_pool))
	{
		printf("fail_2\n");
		return 0;
	}


	/* Initialize Packet Descriptors */
	for(buf = dev->buf_pool, i = 0; i < totoal_pkt_require; i++)
	{
		dev->pkt_pool[i].next = NULL;
		dev->pkt_pool[i].buf = buf;
		buf += PKT_BUFF_SIZE;
	}


	/* Initialize DMA Tx Descriptors */
	for(i = 0; i < dev->num_tx_desc; i++)
	{
		dev->drv_tx_desc[i] = NULL;
  	  	
		dev->hw_tx_desc[i].buf_reg = 0;
		
		dev->hw_tx_desc[i].ctl_reg |= DMA_CPU_OWN;
	}
	
	dev->hw_tx_desc[i - 1].ctl_reg |= DMA_END_OF_RING;


	/* Initialize DMA Rx Descriptors */
	for(i = 0; i < dev->num_rx_desc; i++)
	{
		dev->drv_rx_desc[i] = &dev->pkt_pool[i];
  	
		dev->hw_rx_desc[i].buf_reg = (u_int32)dev->drv_rx_desc[i]->buf;
		
		dev->hw_rx_desc[i].stat_reg = PKT_BUFF_SIZE;
	}
	
	
	dev->hw_rx_desc[i - 1].stat_reg |= DMA_END_OF_RING;


	/* Initialize Free Packet Descriptors */
	dev->free_pkt_list = &dev->pkt_pool[i];
	
	for(; i < (totoal_pkt_require - 1); i++)
	{
		dev->pkt_pool[i].next = &dev->pkt_pool[i + 1];
	}


	/* Initialize index */
	dev->rx_idx = 0;
	
	dev->tx_idx_head = dev->tx_idx_tail = 0;
  
	dev->tx_pkt_queue.head = dev->tx_pkt_queue.tail = NULL;
  
	return 1;
}



int eth_init (bd_t *bd)
{
	orn_device *dev = NULL;


	DBG_FUNC(1);	
	
	ASSERT(!orn_initialized);
	
	if(orn_initialized)
	{
		return 0;
	}
	else
	{
		orn_initialized = 1;
	}
	
	
	dev = (orn_device *)malloc(sizeof(orn_device) + 0x7);
	
	ASSERT(dev);
	
	if(!dev)
	{
		return 0;
	}


	/* 
	 * alignment of the dev info and this might increase the access performance
	 */
	(u32)dev = ((u32)dev + 0x7) & ~0x7;
	
	orn_dev = dev;

	
	/*
	 * read default configuration
	 */
	orn_drv_config(dev);

	
	/*
	 * allocate buffer to tx, rx packets
	 */
	if(!orn_drv_alloc_ring(dev))
		printf("orn_drv_alloc_ring:failed!\n");


	/*
	 * configure orion gigabit switch
	 */
	orn_drv_open(dev);


	return 0;	
}



void eth_halt(void)
{

	DBG_FUNC(1);

	orn_drv_close(orn_dev);
	
	orn_initialized = 0;
	
	orn_dev = NULL;		
}



int eth_rx(void)
{

	DBG_FUNC(4);
	
	ASSERT(orn_dev);	
	
	/* wait for receive interrupt */
	if(!(eth_inl(INT_SRC_REG) & (INT_GSW_FS_RX_CMPLT | INT_GSW_FS_Q_FULL)))
	{
		return 0;
	}
	
	orn_drv_rx(orn_dev);


	/* clear and ack interrupts */
	eth_outl(GSW_INT_STAT_REG, eth_inl(GSW_INT_STAT_REG));
	eth_outl(INT_CLR_REG, (INT_GSW_FS_RX_CMPLT | INT_GSW_FS_Q_FULL));
	
	return 0;
}



int eth_send(volatile void *packet, int length)
{
	drv_pkt_desc *tx_pkt;

	
	DBG_FUNC(1);
	
	ASSERT(orn_dev);
	
	tx_pkt = orn_get_free_pkt(orn_dev);
	
	ASSERT(tx_pkt);
	
	if(!tx_pkt)
	{
	
		return 0;
	}
	

	memcpy(tx_pkt->buf, (void *)packet, length);
	
	if(length < PKT_MIN_LEN)
	{
		length = PKT_MIN_LEN;
	}
	
	tx_pkt->len = length;
	
	/* enqueue tx packet */
	orn_enqueue_tail(&(orn_dev->tx_pkt_queue), tx_pkt);
		
	return orn_drv_send(orn_dev);
}



static int orn_drv_open(orn_device *dev)
{
	u32 volatile    reg;
	u32             mac_hi;
	u16             mac_lo;
	u16		phy0_id = 0, phy1_id = 0;
	u16		phy_reg;
	u8              *mac;
	u32 volatile    i, j;


	DBG_FUNC(1);
	
	ASSERT(dev);



	/* enable the gsw */
	eth_inl(PWR_PLL_DWN_REG) &= ~PWR_PLL_125_DWN;
	eth_inl(PWR_CLK_MSK_REG) &= ~PWR_CLK_GSW_DIS;
	eth_inl(PWR_SOFT_RST_REG) |= PWR_SOFT_RST_GSW;


	/* software reset the gsw */
	eth_inl(PWR_SOFT_RST_REG) |= PWR_SOFT_RST_GSW;
	
	for (i = 0; i < 0x10; i++);
	
	eth_inl(PWR_SOFT_RST_REG) &= ~PWR_SOFT_RST_GSW;
	
	for (i = 0; i < 0x10; i++);
	
	eth_inl(PWR_SOFT_RST_REG) |= PWR_SOFT_RST_GSW;


	/* wait until all embedded memory BIST is complete */
	while (!(eth_inl(GSW_BIST_RSLT_TST_0_REG) & (0x1 << 17)));

	
	/* configure switch */
	dev->ctl_flag = eth_inl(GSW_CFG_REG);
	
	dev->ctl_flag &= ~0xF;		/* disable aging */
	
	dev->ctl_flag |= (GSW_CFG_CRC_STRP | GSW_CFG_MAX_LEN_1536);
	
	dev->ctl_flag &= ~GSW_CFG_IVL;
	
	dev->ctl_flag &= ~GSW_CFG_HNAT_EN;

  	eth_outl(GSW_CFG_REG, dev->ctl_flag);


	/*
	 * Assign PHY address for GSW's MAC port 0/1
	 *
	 * First check if the platform is ICPlus's IP175C platform (dual/single MII-Mode)
	 * Note it seems that ICPlus175C will respond to PHY address 0/1/2/3/4, so
	 * for MAC port 0, the PHY address is set to be above 5
	 */
#ifdef CONFIG_RTL8201CL
	gsw_mac_0_phy_addr = 0x01;//0x0f;
	gsw_mac_1_phy_addr = 0x01;//1f;
#elif defined(CONFIG_DM9161AE)
	gsw_mac_0_phy_addr = 0;//15;
	gsw_mac_1_phy_addr = 0;//4;//4
#else
	gsw_mac_0_phy_addr = 15;
	gsw_mac_1_phy_addr = 4;
#endif

	/*
	 * for STR9100 ECO
	 * Set MAC port 0/1's PHY address for auto polling
	 */
	orn_set_phy_addr(0, gsw_mac_0_phy_addr);
	orn_set_phy_addr(1, gsw_mac_1_phy_addr);

	for (i = 0; i < 0x100; i++)
	{
		// delay some time
		for (j = 0; j < 0x1000; j++);
		
#ifdef CONFIG_RTL8201CL		
		orn_phy_read(gsw_mac_0_phy_addr, 0x3, &phy0_id);
		orn_phy_read(gsw_mac_1_phy_addr, 0x3, &phy1_id);
#else
		orn_phy_read(gsw_mac_0_phy_addr, 0x2, &phy0_id);
		orn_phy_read(gsw_mac_1_phy_addr, 0x2, &phy1_id);
#endif		
		
		if (((phy0_id == 0xFFFF) && (phy1_id == 0xFFFF))||
		    ((phy0_id == 0x0) && (phy1_id == 0x0)))
			continue;
		else
			break;		
	}
	
	if (((phy0_id == 0xFFFF) && (phy1_id == 0xFFFF)) ||
	    ((phy0_id == 0x0) && (phy1_id == 0x0)))
	{
		gsw_mac_0_phy_addr = 0;
		gsw_mac_1_phy_addr = 1;

		/*
		 * for STR9100 ECO
		 * Set MAC port 0/1's PHY address for auto polling
	 	*/
		orn_set_phy_addr(0, gsw_mac_0_phy_addr);
		orn_set_phy_addr(1, gsw_mac_1_phy_addr);
		

		for (i = 0; i < 0x100; i++)
		{
			// delay some time
			for (j = 0; j < 0x1000; j++);
		
#ifdef CONFIG_RTL8201CL		
			orn_phy_read(gsw_mac_0_phy_addr, 0x3, &phy0_id);
			orn_phy_read(gsw_mac_1_phy_addr, 0x3, &phy1_id);
#else
			orn_phy_read(gsw_mac_0_phy_addr, 0x2, &phy0_id);
			orn_phy_read(gsw_mac_1_phy_addr, 0x2, &phy1_id);
#endif	
		
			if (((phy0_id == 0xFFFF) && (phy1_id == 0xFFFF)) ||
		    	    ((phy0_id == 0x0) && (phy1_id == 0x0)))
				continue;
			else
				break;		
		}
	}

	printf("\n PHY:0x%x, 0x%x\n", phy0_id, phy1_id); //alan

	if ((phy0_id == 0xFFFF) && (phy1_id == 0x000F))
    	{
        	orn_configure_asix_gsw_cicada_phy();
    	}
    	else if ((phy0_id == 0xFFFF) && (phy1_id == 0x0282))
    	{    		
    		orn_configure_asix_gsw_agere_phy();
    	}
    	else if ((phy0_id == 0x0243) && (phy1_id == 0x0243))
    	{ 
        	orn_configure_icplus_phy_icplus_phy();
    	}
        else if ((phy0_id == 0xFFFF) && (phy1_id == 0x0243))
        {
		/*
		 * PHY address 29?????
		 */
		orn_phy_read(29, 23, &phy_reg);

	
		if ((phy_reg & 0x1F) == 0)
		{
			// ICPlus175C in dual MII modes
			orn_icplus175c_mii_mode = 0;
		}
		else
		{
			// ICPlus175C in signal MII mode
			orn_icplus175c_mii_mode = 1;
		}
		
                orn_configure_icplus_175c_phy4();
        }
#ifdef CONFIG_RTL8201CL
        else if((phy0_id == 0x8201) && (phy1_id == 0x8201))
		{
			//printf("RTL8201 Found.!\n");
			orn_configure_rtl8201cl_phy();
			
		}
#endif
#ifdef CONFIG_DM9161AE
	else if((phy0_id == 0x0181) || (phy1_id == 0x0181))
		{
			//printf("RTL8201 Found.!\n");
			orn_configure_dm9161ae_phy();
			
		}
#endif
	else
        {
		printf("\nSomething wrong with the hardware platform for incorrect PHY_ID!!\n");
		
		return -1;
        }


	/* configure cpu port */
	reg = eth_inl(GSW_CPU_PORT_CFG_REG);

	// disable CPU port SA learning
	reg |= GSW_CPU_PORT_SA_LRN_DIS;
	
	// enable CPU port
	reg &= ~GSW_CPU_PORT_DIS;



	/*
	 * for STR9100 ECO
	 * Disable RXDMA supporting offset 2 bytes
	 * Note :
	 * 0 : for RXDMA (4N + 2) mode
	 * 1 : for RXDMA (4N) mode
	 */	
	reg |= ((u32)0x1 << 31);

	eth_outl(GSW_CPU_PORT_CFG_REG, reg);



	/* configure pvid */
	reg = GSW_VLAN_PVID(CPU, dev->port[2].pvid) | 
	      GSW_VLAN_PVID(0, dev->port[0].pvid) | 
	      GSW_VLAN_PVID(1, dev->port[1].pvid);
       
	eth_outl(GSW_VLAN_PVID_REG, reg);

	
	/* configure vlan group id */
	reg = ((dev->vlan[0].vid << GSW_VLAN_ID(0)) | (dev->vlan[1].vid << GSW_VLAN_ID(1)));
	
	eth_outl(GSW_VLAN_VID_0_1_REG, reg);


#if 0	
	// for testing only
        if (orn_icplus175c_mii_mode)
        {
		dev->vlan[2].vid = 2;
        }
#endif

	reg = ((dev->vlan[2].vid << GSW_VLAN_ID(2)) | (dev->vlan[3].vid << GSW_VLAN_ID(3)));
	
	eth_outl(GSW_VLAN_VID_2_3_REG, reg);

	
	reg = ((dev->vlan[4].vid << GSW_VLAN_ID(4)) | (dev->vlan[5].vid << GSW_VLAN_ID(5)));
	
	eth_outl(GSW_VLAN_VID_4_5_REG, reg);


	reg = ((dev->vlan[6].vid << GSW_VLAN_ID(6)) | (dev->vlan[7].vid << GSW_VLAN_ID(7)));
	
	eth_outl(GSW_VLAN_VID_6_7_REG, reg);	


#if 0
	// for testing only
        if (orn_icplus175c_mii_mode)
        {
		dev->vlan[2].port_map |= (1 << 0);
        }
#endif

	/* configure VLAN port map */
	for (reg = 0, i = 0; i < 8; i++)
	{
		reg |= GSW_VLAN_PORT_MAP(i, dev->vlan[i].port_map);
	}
	
	eth_outl(GSW_VLAN_MBR_PMAP_REG, reg);


        
        if (orn_icplus175c_mii_mode)
        {
		/*
		 * ICPlus175C is in single MII mode; packets outgoing from GSW's MAC port 0
		 * will be instructed to be tagged
		 */
		dev->vlan[1].tag_port_map |= (1 << 0);
        }
        

	/* configure vlan tag port map */
	for (reg = 0, i = 0; i < 8; i++)
	{
		reg |= GSW_VLAN_PORT_MAP(i, dev->vlan[i].tag_port_map);
	}
	
	eth_outl(GSW_VLAN_TAG_PMAP_REG, reg);




	/* 
	 * configure Default ARL Entry
	 */
	for (i = 0; i < 3; i++)
	{
		/* parse mac address */
		mac = (u8 *)dev->vlan[i].mac;
		
		mac_lo = ((mac[1] << 8) | mac[0]);
		
		mac_hi = ((mac[5] << 24) | (mac[4] << 16) | (mac[3] << 8) | mac[2]);

		
		reg = 0;
		
		eth_outl(GSW_ARL_TBL_ACCTL_0_REG, reg);
	
		reg = ((GSW_ARL_VLAN_MAC) | 
		       (dev->vlan[i].gid << GSW_ARL_VLAN_ID) | 
		       (0x7 << GSW_ARL_AGE_FLD) | 
		       (dev->vlan[i].port_map << GSW_ARL_PORT_MAP) | 
		       (mac_lo << GSW_ARL_MAC_LO));
	
		eth_outl(GSW_ARL_TBL_ACCTL_1_REG, reg);

		reg = mac_hi;
		
		eth_outl(GSW_ARL_TBL_ACCTL_2_REG, reg);


		/* write entry and wait */
		eth_outl(GSW_ARL_TBL_ACCTL_0_REG, GSW_ARL_CMD_WRITE);


		for (j = 0; !(eth_inl(GSW_ARL_TBL_ACCTL_1_REG) & GSW_ARL_CMD_CMPLT); j++)
		{
			if (j >= ARL_CMD_WAIT)
			{
				printf("\n\ntime-out for writing ARL table entry!!\n\n");
				
				return orn_drv_close(dev);
  			}
  		
			udelay(1000000);
		}
	}


	/* initialized interrupt */
	eth_inl(INT_TRGR_MODE_REG) &= ~INT_GSW_STAT;
	
	eth_inl(INT_TRGR_MODE_REG) |= (INT_GSW_TS_TX_CMPLT |
				       INT_GSW_FS_RX_CMPLT |
				       INT_GSW_TS_Q_EMPTY |
				       INT_GSW_FS_Q_FULL);
	
	eth_inl(INT_TRGR_LVL_REG) &= (~INT_GSW_STAT & 
				      ~INT_GSW_FS_RX_CMPLT & 
				      ~INT_GSW_TS_Q_EMPTY & 
				      ~INT_GSW_FS_Q_FULL & 
				      ~INT_GSW_STAT);

#if 0
	/*
	 * all interrupt sources should already be disables by start.S
	 */
	/* disable all GSW status interrupts */
	eth_outl(GSW_INT_MSK_REG, GSW_INT_MSK_ALL);

	/* disable DMA Interrupts */
	reg = eth_inl(INT_MSK_REG);
	
	reg |= (INT_GSW_TS_TX_CMPLT | INT_GSW_FS_RX_CMPLT | INT_GSW_TS_Q_EMPTY | INT_GSW_FS_Q_FULL);
	
	eth_outl(INT_MSK_REG, reg);	
#endif		

	/* clear all interrupts */
	eth_outl(GSW_INT_STAT_REG, GSW_INT_MSK_ALL);
	
	
	/* clear DMA Interrupts */
	reg = (INT_GSW_TS_TX_CMPLT | INT_GSW_FS_RX_CMPLT | INT_GSW_TS_Q_EMPTY | INT_GSW_FS_Q_FULL);
	
	eth_outl(INT_CLR_REG, reg);

	
	/* stop all DMA operation */
	eth_outl(GSW_TS_DMA_CTL_REG, GSW_DMA_DIS);
	eth_outl(GSW_FS_DMA_CTL_REG, GSW_DMA_DIS);


	/* configure DMA descriptors */
	eth_outl(GSW_TS_DESC_PTR_REG,   (u32)dev->hw_tx_desc);
	eth_outl(GSW_TS_DESC_BASE_ADDR, (u32)dev->hw_tx_desc);
	eth_outl(GSW_FS_DESC_PTR_REG,   (u32)dev->hw_rx_desc);
	eth_outl(GSW_FS_DESC_BASE_ADDR, (u32)dev->hw_rx_desc);

	/* start rx DMA operation */
	eth_outl(GSW_FS_DMA_CTL_REG, GSW_DMA_EN);


	return 0;	
}



/******************************************************************************
 *
 * FUNCTION:  orn_configure_asix_gsw_cicada_phy
 * PURPOSE:
 *
 ******************************************************************************/
static void orn_configure_asix_gsw_cicada_phy(void)
{
	u32 volatile    reg;
	u16		phy_reg;
	u32 volatile    i, j;

	
	/*
	 * Configure GSW's MAC port 0
	 * For ASIX's 5-port GbE Switch setting
	 * 1. No SMI (MDC/MDIO) connection between Orion's MAC port 0 and ASIX's MAC port 4
	 * 2. Force Orion's MAC port 0 to be 1000Mbps, and full-duplex, and flow control on
	 */
	reg = eth_inl(GSW_PORT0_CFG_REG);

	// disable PHY's AN
	reg &= ~(0x1 << 7);

	// enable RGMII-PHY mode
	reg |= (0x1 << 15);

	// force speed = 1000Mbps
	reg &= ~(0x3 << 8);
	reg |= (0x2 << 8);
    
	// force full-duplex
	reg |= (0x1 << 10);

	// force Tx/Rx flow-control on
	reg |= (0x1 << 11) | (0x1 << 12);

	eth_outl(GSW_PORT0_CFG_REG, reg);


	/*
	 * delay some time??????
	 */
	for (i = 0; i < 0x2000; i++);


	printf("\rCheck MAC/PHY 0 Link Status : ");


	reg = eth_inl(GSW_PORT0_CFG_REG);
	
	if (((reg & 0x1) == 0) || (reg & 0x2))
	{
		/*
		 * Port 0 PHY link down or no TXC in Port 0
		 */
		printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
	}
	else
	{
		// enable the forwarding of broadcast packets to CPU
		reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));

		// disable the forwarding of unknown, multicast packets to CPU
		reg |= ((0x1 << 25) | (0x1 << 26));

		// include unknown, multicast and broadcast packets into broadcast storm
		reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));

		// enable GSW MAC port 0
		reg &= ~(0x1 << 18);
		
		eth_outl(GSW_PORT0_CFG_REG, reg);
		
		printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
	}


	/*
	 * Configure MAC port 1
	 * For Cicada CIS8201 single PHY
	 */
	reg = eth_inl(GSW_PORT1_CFG_REG);
    
	// disable PHY's AN
	reg &= ~(0x1 << 7);
    
	// enable RGMII-PHY mode
	reg |= (0x1 << 15);

	// enable GSW MAC port 1
	reg &= ~(0x1 << 18);

	eth_outl(GSW_PORT1_CFG_REG, reg);


	/*
	 * Configure Cicada's CIS8201 single PHY
	 */
	orn_phy_read(gsw_mac_1_phy_addr, 0x1C, &phy_reg);
    
	// configure SMI registers have higher priority over MODE/FRC_DPLX, and ANEG_DIS pins
	phy_reg |= (0x1 << 2);
    
	orn_phy_write(gsw_mac_1_phy_addr, 0x1C, phy_reg);



	orn_phy_read(gsw_mac_1_phy_addr, 0x17, &phy_reg);
    
	// enable RGMII MAC interface mode
	phy_reg &= ~(0xF << 12);
	phy_reg |= (0x1 << 12);	
    
	// enable RGMII I/O pins operating from 2.5V supply
	phy_reg &= ~(0x7 << 9);
	phy_reg |= (0x1 << 9);
    
	orn_phy_write(gsw_mac_1_phy_addr, 0x17, phy_reg);


	orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_reg);
    
	// Enable symmetric Pause capable
	phy_reg |= (0x1 << 10);
    
	orn_phy_write(gsw_mac_1_phy_addr, 0x4, phy_reg);


	reg = eth_inl(GSW_PORT1_CFG_REG);
    
	// enable PHY's AN
	reg |= (0x1 << 7);

	eth_outl(GSW_PORT1_CFG_REG, reg);


	/*
	 * Enable PHY1 AN restart bit to restart PHY1 AN
	 */
	orn_phy_read(gsw_mac_1_phy_addr, 0x0, &phy_reg);
        
	phy_reg |= (0x1 << 9) | (0x1 << 12);
        
	orn_phy_write(gsw_mac_1_phy_addr, 0x0, phy_reg);


	/*
	 * Polling until PHY1 AN restart is complete
	 */
	for (i = 0; i < 0x300; i++)
	{
		orn_phy_read(gsw_mac_1_phy_addr, 0x1, &phy_reg);
        
		if ((phy_reg & (0x1 << 5)) && (phy_reg & (0x1 << 2)))
		{
			break;
		}
		else
		{		
			for (j = 0; j < 0x1000; j++);
			
			
			if ((i % 4) == 0)
				printf("\rCheck MAC/PHY 1 Link Status : |");
			else if ((i % 4) == 1)
				printf("\rCheck MAC/PHY 1 Link Status : /");
			else if ((i % 4) == 2)
				printf("\rCheck MAC/PHY 1 Link Status : -");
			else if ((i % 4) == 3)
				printf("\rCheck MAC/PHY 1 Link Status : \\");
		}
	}

	/*
	 * delay some time??????
	 */
	for (i = 0; i < 0x2000; i++);


	reg = eth_inl(GSW_PORT1_CFG_REG);
    
	if (((reg & 0x1) == 0) || (reg & 0x2))
	{
		/*
		 * Port 1 PHY link down or no TXC in Port 1        
		 */
		printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
	}
	else
	{
		/*
		 * enable the forwarding of unknown, multicast and broadcast packets to CPU
		 */
		reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));      
        
		/*
		 * include unknown, multicast and broadcast packets into broadcast storm
		 */
		reg |= ((0x1 << 29) | (0x1 << 30) | ((u32)0x1 << 31));
    
		eth_outl(GSW_PORT1_CFG_REG, reg);
		
		printf("\rCheck MAC/PHY 1 Link Status : UP!\n");
		
		
		/*
		 * There is a bug for CIS8201 PHY operating at 10H mode, and we use the following 
		 * code segment to work-around
		 */
		orn_phy_read(gsw_mac_1_phy_addr, 0x05, &phy_reg);
	
		if ((phy_reg & (0x1 << 5)) && 
	    	    (!(phy_reg & (0x1 << 6))) &&
	            (!(phy_reg & (0x1 << 7))) &&
	            (!(phy_reg & (0x1 << 8)))) /* 10H,10F/100F/100H off */
		{
  	    		orn_phy_read(gsw_mac_1_phy_addr, 0x0a, &phy_reg);
  	    
	    		if ((!(phy_reg & (0x1 << 10))) && (!(phy_reg & (0x1 << 11)))) /* 1000F/1000H off */
	    		{
	        		orn_phy_read(gsw_mac_1_phy_addr, 0x16, &phy_reg);
	        
	        		phy_reg |= (0x1 << 13) | (0x1 << 15);  // disable "Link integrity check(B13)" & "Echo mode(B15)"
	        
	        		orn_phy_write(gsw_mac_1_phy_addr, 0x16, phy_reg);	        
	        
	        		//printf("\n*MAC port 1 :disable \"Link integrity check(B13)\" & \"Echo mode(B15)\"   \n"); 
	    		}
		}
	}


	/*
	 * Adjust MAC port 0/1 TX/RX skew     
	 */
	// adjust MAC port 0 RX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 24);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 24);


	// adjust MAC port 0 TX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 26);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 26);


	// adjust MAC port 1 RX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 28);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 28);


	// adjust MAC port 1 TX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 30);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 30);


	/*
	 * Configure MAC port 0/1 pad drive strength
	 */
	// configure MAC port 0 pad drive strength = 10/100 mode
	*(u_int32 volatile *)(0x7700001C) |= (0x1 << 2);
}



/******************************************************************************
 *
 * FUNCTION:  orn_configure_asix_gsw_agere_phy
 * PURPOSE:
 *
 ******************************************************************************/
static void orn_configure_asix_gsw_agere_phy(void)
{
	u32 volatile    reg;
	u16		phy_reg;
	u32 volatile    i, j;

	
	/*
	 * Configure GSW's MAC port 0
	 * For ASIX's 5-port GbE Switch setting
	 * 1. No SMI (MDC/MDIO) connection between Orion's MAC port 0 and ASIX's MAC port 4
	 * 2. Force Orion's MAC port 0 to be 1000Mbps, and full-duplex, and flow control on
	 */
	reg = eth_inl(GSW_PORT0_CFG_REG);

	// disable PHY's AN
	reg &= ~(0x1 << 7);

	// enable RGMII-PHY mode
	reg |= (0x1 << 15);

	// force speed = 1000Mbps
	reg &= ~(0x3 << 8);
	reg |= (0x2 << 8);
    
	// force full-duplex
	reg |= (0x1 << 10);

	// force Tx/Rx flow-control on
	reg |= (0x1 << 11) | (0x1 << 12);

	eth_outl(GSW_PORT0_CFG_REG, reg);


	/*
	 * delay some time??????
	 */
	for (i = 0; i < 0x2000; i++);


	printf("\rCheck MAC/PHY 0 Link Status : ");


	reg = eth_inl(GSW_PORT0_CFG_REG);
	
	if (((reg & 0x1) == 0) || (reg & 0x2))
	{
		/*
		 * Port 0 PHY link down or no TXC in Port 0
		 */
		printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
	}
	else
	{
		// enable the forwarding of broadcast packets to CPU
		reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));

		// disable the forwarding of unknown, multicast packets to CPU
		reg |= ((0x1 << 25) | (0x1 << 26));

		// include unknown, multicast and broadcast packets into broadcast storm
		reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));

		// enable GSW MAC port 0
		reg &= ~(0x1 << 18);
		
		eth_outl(GSW_PORT0_CFG_REG, reg);
		
		printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
	}


	/*
	 * Configure GSW MAC port 1
	 * For Agere Systems's ET1011 single PHY
	 */
	reg = eth_inl(GSW_PORT1_CFG_REG);
	
	// enable GSW MAC port 1
	reg &= ~(0x1 << 18);
    
	// disable AN bit of GSW's MAC port 1
	reg &= ~(0x1 << 7);
    
	// enable RGMII-PHY mode
	reg |= (0x1 << 15);

	eth_outl(GSW_PORT1_CFG_REG, reg);


	/*
	 * Configure Agere Systems's ET1011 single PHY
	 * Note there is CRC error bug in Agere's ET1011, so we have to patch it first
	 */
	orn_configure_agere_phy_patch(gsw_mac_1_phy_addr);
	

	orn_phy_read(gsw_mac_1_phy_addr, 22, &phy_reg);
    
	// enable RGMII MAC interface mode : RGMII/RMII (trace delay) mode
	phy_reg &= ~(0x7 << 0);
	phy_reg |= (0x4 << 0);		// RGMII/RMII trace delay mode
    
	orn_phy_write(gsw_mac_1_phy_addr, 22, phy_reg);


	// Enable symmetric Pause capable
	orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_reg);
    
	phy_reg |= (0x1 << 10);
    
	orn_phy_write(gsw_mac_1_phy_addr, 0x4, phy_reg);


	// enable AN bit of GSW's MAC port 1 
	reg = eth_inl(GSW_PORT1_CFG_REG);
    
	reg |= (0x1 << 7);

	eth_outl(GSW_PORT1_CFG_REG, reg);



	/*
	 * Enable PHY1 AN restart bit to restart PHY1 AN
	 */
	orn_phy_read(gsw_mac_1_phy_addr, 0x0, &phy_reg);
        
	phy_reg |= (0x1 << 9) | (0x1 << 12);
        
	orn_phy_write(gsw_mac_1_phy_addr, 0x0, phy_reg);


	/*
	 * Polling until PHY1 AN restart is complete and PHY1 link status is UP
	 */
	for (i = 0; i < 0x400; i++)
	{
		orn_phy_read(gsw_mac_1_phy_addr, 0x1, &phy_reg);
        
		if ((phy_reg & (0x1 << 5)) && (phy_reg & (0x1 << 2)))
		{
			break;
		}
		else
		{		
			for (j = 0; j < 0x1000; j++);
			
			
			if ((i % 4) == 0)
				printf("\rCheck MAC/PHY 1 Link Status : |");
			else if ((i % 4) == 1)
				printf("\rCheck MAC/PHY 1 Link Status : /");
			else if ((i % 4) == 2)
				printf("\rCheck MAC/PHY 1 Link Status : -");
			else if ((i % 4) == 3)
				printf("\rCheck MAC/PHY 1 Link Status : \\");
		}
	}

	/*
	 * delay some time??????
	 */
	for (i = 0; i < 0x2000; i++);


	reg = eth_inl(GSW_PORT1_CFG_REG);
    
	if (((reg & 0x1) == 0) || (reg & 0x2))
	{
		/*
		 * Port 1 PHY link down or no TXC in Port 1        
		 */
		printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
	}
	else
	{
		/*
		 * enable the forwarding of unknown, multicast and broadcast packets to CPU
		 */
		reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));      
        
		/*
		 * include unknown, multicast and broadcast packets into broadcast storm
		 */
		reg |= ((0x1 << 29) | (0x1 << 30) | ((u32)0x1 << 31));
    
		eth_outl(GSW_PORT1_CFG_REG, reg);
		
		printf("\rCheck MAC/PHY 1 Link Status : UP!\n");
	}


	/*
	 * Adjust MAC port 0/1 TX/RX skew     
	 */
	// adjust MAC port 0 RX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 24);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 24);


	// adjust MAC port 0 TX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 26);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 26);


	// adjust MAC port 1 RX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 28);
	*(u_int32 volatile *)(0x70000094) |= (0x2 << 28);


	// adjust MAC port 1 TX skew
	*(u_int32 volatile *)(0x70000094) &= ~(0x3 << 30);
	*(u_int32 volatile *)(0x70000094) |= (0x3 << 30);


	/*
	 * Configure MAC port 0/1 pad drive strength
	 */
	// configure MAC port 0 pad drive strength = 10/100 mode
	*(u_int32 volatile *)(0x7700001C) |= (0x1 << 2);
}



/******************************************************************************
 *
 * FUNCTION:  orn_configure_agere_phy_patch
 * PURPOSE:
 *
 ******************************************************************************/
void orn_configure_agere_phy_patch(u8 phy_addr)
{
	u16	phy_data = 0;


	/*
	* Patch for CRC error based on Agere's programming note for ET1011
	*/
	//1. power-down the PHY
	orn_phy_read(phy_addr, 0, &phy_data);
	phy_data |= (0x1 << 11);
	orn_phy_write(phy_addr, 0, phy_data);


	//2. Enable PHY programming mode
	orn_phy_read(phy_addr, 18, &phy_data); 
	phy_data |= (0x1 << 0);
	phy_data |= (0x1 << 2);
	orn_phy_write(phy_addr, 18, phy_data);


	//3. Perform some PHY register with the Agere-specfic value
	orn_phy_write(phy_addr, 16, 0x880e);    
	orn_phy_write(phy_addr, 17, 0xb4d3);    
    
	orn_phy_write(phy_addr, 16, 0x880f);    
	orn_phy_write(phy_addr, 17, 0xb4d3);    
   
	orn_phy_write(phy_addr, 16, 0x8810);    
	orn_phy_write(phy_addr, 17, 0xb4d3);    
    
	orn_phy_write(phy_addr, 16, 0x8817);    
	orn_phy_write(phy_addr, 17, 0x1c00);    
    
	orn_phy_write(phy_addr, 16, 0x8805);    
	orn_phy_write(phy_addr, 17, 0xb03e);    
    
	orn_phy_write(phy_addr, 16, 0x8806);    
	orn_phy_write(phy_addr, 17, 0xb03e);    

	orn_phy_write(phy_addr, 16, 0x8807);
	orn_phy_write(phy_addr, 17, 0xff00);    

	orn_phy_write(phy_addr, 16, 0x8808);    
	orn_phy_write(phy_addr, 17, 0xe110);    

	orn_phy_write(phy_addr, 16, 0x300d);    
	orn_phy_write(phy_addr, 17, 0x0001);    


	//4. Disable PHY programming mode
	orn_phy_read(phy_addr, 18, &phy_data); 
	phy_data &= ~(0x1 << 0);
	phy_data &= ~(0x1 << 2);
	orn_phy_write(phy_addr, 18, phy_data);   


	//5. power-up the PHY
	orn_phy_read(phy_addr, 0, &phy_data); 
	phy_data &= ~(0x1 << 11);
	orn_phy_write(phy_addr, 0, phy_data);
}



/******************************************************************************
 *
 * FUNCTION:  orn_configure_icplus_phy_icplus_phy
 * PURPOSE:
 *
 ******************************************************************************/
static void orn_configure_icplus_phy_icplus_phy(void)
{
	u32 volatile    reg;
	u16		phy_reg;
	u32 volatile    i, j;


	/* 
	 * configure port 0 
	 */
	printf("\rCheck MAC/PHY 0 Link Status : ");

	reg = eth_inl(GSW_PORT0_CFG_REG);

	if ((reg & (0x1 << 0)) && !(reg & (0x1 << 1)))
  	{
  		reg &= (~GSW_PORT_FWD_UNK_DIS & ~GSW_PORT_FWD_MC_DIS & ~GSW_PORT_FWD_BC_DIS);

		reg |= (GSW_PORT_FWD_UNK_DIS | GSW_PORT_FWD_MC_DIS);

  		reg |= (GSW_PORT_BCS_UNK_EN | GSW_PORT_BCS_MC_EN | GSW_PORT_BCS_BC_EN);
  	
		/*
		 * enable GSW MAC port 0
		 */
  		reg &= ~(0x1 << 18);
  		
  		eth_outl(GSW_PORT0_CFG_REG, reg);


  		/* enable flow control on (PAUSE frame) */
		orn_phy_read(gsw_mac_0_phy_addr, 0x4, &phy_reg);
		
  		phy_reg |= (0x1 << 10);
  		
	  	orn_phy_write(gsw_mac_0_phy_addr, 0x4, phy_reg);
  	
		
  		/* restart PHY auto neg. */
		orn_phy_read(gsw_mac_0_phy_addr, PHY_CTL_REG, &phy_reg);
		
  		phy_reg |= (0x1 << 9) | (0x1 << 12);
  		
	  	orn_phy_write(gsw_mac_0_phy_addr, PHY_CTL_REG, phy_reg);


		/* wait for PHY auto neg. complete */
		for (i = 0; i < 0x300; i++)
		{
  			orn_phy_read(gsw_mac_0_phy_addr, 0x1, &phy_reg);
  			
			if ((phy_reg & (0x1 << 2)) && (phy_reg & (0x1 << 5)))
			{
				break;
			}
			else
			{
				for (j = 0; j < 0x1000; j++);			
			
				if ((i % 4) == 0)
					printf("\rCheck MAC/PHY 0 Link Status : |");
				else if ((i % 4) == 1)
					printf("\rCheck MAC/PHY 0 Link Status : /");
				else if ((i % 4) == 2)
					printf("\rCheck MAC/PHY 0 Link Status : -");
				else if ((i % 4) == 3)
					printf("\rCheck MAC/PHY 0 Link Status : \\");
			}
		}


		if (i >= 0x300)
		{
			printf("\rCheck MAC/PHY 0 Link Status : Restart AN Uncompleted!\n");
		}
		else
		{
			while (!(eth_inl(GSW_PORT0_CFG_REG) & GSW_PORT_LNK_UP))
			{
				/* wait for link up */
				printf("\rCheck MAC/PHY 0 Link Status : |");
				printf("\rCheck MAC/PHY 0 Link Status : /");
				printf("\rCheck MAC/PHY 0 Link Status : -");
				printf("\rCheck MAC/PHY 0 Link Status : \\");
			}

			printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
		}
  	}
	else
	{
		printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
	}


  	/*
  	 * configure port 1
  	 */
	printf("\rCheck MAC/PHY 1 Link Status : ");

  	reg = eth_inl(GSW_PORT1_CFG_REG);

	if ((reg & (0x1 << 0)) && !(reg & (0x1 << 1)))
  	{
  		reg &= (~GSW_PORT_FWD_UNK_DIS & ~GSW_PORT_FWD_MC_DIS & ~GSW_PORT_FWD_BC_DIS);

		reg |= (GSW_PORT_FWD_UNK_DIS | GSW_PORT_FWD_MC_DIS);

  		reg |= (GSW_PORT_BCS_UNK_EN | GSW_PORT_BCS_MC_EN | GSW_PORT_BCS_BC_EN);

 
 		/*
		 * enable GSW MAC port 1
		 */
  		reg &= ~(0x1 << 18);
  		
  		eth_outl(GSW_PORT1_CFG_REG, reg);


  		/* enable flow control on */
		orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_reg);
		
  		phy_reg |= (0x1 << 10);
  		
	  	orn_phy_write(gsw_mac_1_phy_addr, 0x4, phy_reg);


	  	/* restart PHY auto neg. */
		orn_phy_read(gsw_mac_1_phy_addr, PHY_CTL_REG, &phy_reg);
		
  		phy_reg |= (0x1 << 9) | (0x1 << 12);
		
		orn_phy_write(gsw_mac_1_phy_addr, PHY_CTL_REG, phy_reg);


		/* wait for PHY auto neg. complete */
		for (i = 0; i < 0x300; i++)
		{
  			orn_phy_read(gsw_mac_1_phy_addr, 0x1, &phy_reg);
  			
			if ((phy_reg & (0x1 << 2)) && (phy_reg & (0x1 << 5)))
			{
				break;
			}
			else
			{
				for (j = 0; j < 0x1000; j++);			
			
				if ((i % 4) == 0)
					printf("\rCheck MAC/PHY 1 Link Status : |");
				else if ((i % 4) == 1)
					printf("\rCheck MAC/PHY 1 Link Status : /");
				else if ((i % 4) == 2)
					printf("\rCheck MAC/PHY 1 Link Status : -");
				else if ((i % 4) == 3)
					printf("\rCheck MAC/PHY 1 Link Status : \\");
			}
		}		
		
		if (i >= 0x300)
		{
			printf("\rCheck MAC/PHY 1 Link Status : Restart AN Uncompleted!\n");
		}
		else
		{
			while (!(eth_inl(GSW_PORT1_CFG_REG) & GSW_PORT_LNK_UP))
			{
				/* wait for link up */
				printf("\rCheck MAC/PHY 1 Link Status : |");
				printf("\rCheck MAC/PHY 1 Link Status : /");
				printf("\rCheck MAC/PHY 1 Link Status : -");
				printf("\rCheck MAC/PHY 1 Link Status : \\");
			}

			printf("\rCheck MAC/PHY 1 Link Status : UP!\n");
		}
	}
	else
	{
		printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
	}	
}



/******************************************************************************
 *
 * FUNCTION:  orn_configure_icplus_175c_phy4
 * PURPOSE:
 *
 ******************************************************************************/
void orn_configure_icplus_175c_phy4(void)
{
    u16			phy_data = 0, phy_data2 = 0;
    u32 volatile	ii, jj;
    u8			phy_speed_dup = 0, phy_flowctrl = 0;
    u32 volatile	reg;


    /*
     * Configure MAC port 0
     * For IP175C Switch setting
     * Force 100Mbps, and full-duplex, and flow control on
     */
    reg = eth_inl(GSW_PORT0_CFG_REG);

    // disable PHY's AN
    reg &= ~(0x1 << 7);

    // disable RGMII-PHY mode
    reg &= ~(0x1 << 15);

    // force speed = 100Mbps
    reg &= ~(0x3 << 8);
    reg |= (0x1 << 8);
    
    // force full-duplex
    reg |= (0x1 << 10);

    // force Tx/Rx flow-control on
    reg |= (0x1 << 11) | (0x1 << 12);

    eth_outl(GSW_PORT0_CFG_REG, reg);


    for (ii = 0; ii < 0x2000; ii++)
    {
        reg = eth_inl(GSW_PORT0_CFG_REG);
    	
        if ((reg & 0x1) && !(reg & 0x2))
        {
            /*
             * enable MAC port 0
             */
            reg &= ~(0x1 << 18);

           
            /*
             * enable the forwarding of unknown, multicast and broadcast packets to CPU
             */
            reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));
        
            /*
             * include unknown, multicast and broadcast packets into broadcast storm
             */
            reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));
            
            eth_outl(GSW_PORT0_CFG_REG, reg);
            
            break;
        }
        else
        {
            for (jj = 0; jj < 0x1000; jj++);
			
			
            if ((ii % 4) == 0)
                printf("\rCheck MAC/PHY 0 Link Status : |");
            else if ((ii % 4) == 1)
                printf("\rCheck MAC/PHY 0 Link Status : /");
            else if ((ii % 4) == 2)
                printf("\rCheck MAC/PHY 0 Link Status : -");
            else if ((ii % 4) == 3)
                printf("\rCheck MAC/PHY 0 Link Status : \\");
        }
    }


    if (!(reg & 0x1) || (reg & 0x2))
    {
        /*
         * Port 0 PHY link down or no TXC in Port 0
         */
        printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
        
        return;
    }
    else
    {
        printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
    }



    /*
     * Configure MAC port 1
     */
    reg = eth_inl(GSW_PORT1_CFG_REG);
    
    // disable MAC's AN
    reg &= ~(0x1 << 7);

    eth_outl(GSW_PORT1_CFG_REG, reg);


    /* enable flow control on (PAUSE frame) */
    orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_data);
		
    phy_data |= (0x1 << 10);  		

    orn_phy_write(gsw_mac_1_phy_addr, 0x4, phy_data);


    /* restart PHY auto neg. */
    orn_phy_read(gsw_mac_1_phy_addr, 0x0, &phy_data);
		
    phy_data |= (0x1 << 9) | (0x1 << 12);		

    orn_phy_write(gsw_mac_1_phy_addr, 0x0, phy_data);



    /* wait for PHY auto neg. complete */
    for (ii = 0; ii < 0x20; ii++)
    {
        orn_phy_read(gsw_mac_1_phy_addr, 0x1, &phy_data);
  			
        if ((phy_data & (0x1 << 2)) && (phy_data & (0x1 << 5)))
        {
            break;
        }
        else
        {			
            if ((ii % 4) == 0)
                printf("\rCheck MAC/PHY 1 Link Status : |");
            else if ((ii % 4) == 1)
                printf("\rCheck MAC/PHY 1 Link Status : /");
            else if ((ii % 4) == 2)
                printf("\rCheck MAC/PHY 1 Link Status : -");
            else if ((ii % 4) == 3)
                printf("\rCheck MAC/PHY 1 Link Status : \\");
        }
    }		


    if (ii >= 0x20)
    {
        printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
        
        return;
    }
    else
    {
        printf("\rCheck MAC/PHY 1 Link Status : UP!\n");
    }


    orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_data);

    orn_phy_read(gsw_mac_1_phy_addr, 0x5, &phy_data2);


    if (phy_data & 0x0400)	//FC on
    {
        //printf("<FC ON>");
        phy_flowctrl = 1;
    }
    else    
    {
        // printf("<FC OFF>");
        phy_flowctrl = 0;
    }    
    
    
    phy_speed_dup = 0;
    
    if ((phy_data & 0x0100) && (phy_data2 & 0x0100)) //100F
    {
        // printf("<100F>");
        phy_speed_dup |= (0x1 << 3); //set bit3 for 100F
    }
    else if ((phy_data & 0x0080) && (phy_data2 & 0x0080)) //100F
    {
        // printf("<100H>");
        phy_speed_dup |= (0x1 << 2);
    }
    else if ((phy_data & 0x0040) && (phy_data2 & 0x0040)) //100F
    {
        // printf("<10F>");
        phy_speed_dup |= (0x1 << 1);
    }
    else if ((phy_data & 0x0020) && (phy_data2 & 0x0020)) //100F
    {
        // printf("<10H>");
        phy_speed_dup |= 0x1;    
    }


    /*
     * Configure MAC port 1 in forced setting subject to the current PHY status
     */     
    reg = eth_inl(GSW_PORT1_CFG_REG);
     
    reg &= ~(0x1 << 7);	//AN off
            
    reg &= ~(0x3 << 8);
            
    if (phy_speed_dup & 0x0C)     //100
    {          
        //printf("<set 100>");
        reg |= (0x01 << 8);              
    }
    else if (phy_speed_dup & 0x03)     //10          
    {
        //printf("<set 10>");
        reg |= (0x00 << 8);
    }
           
    reg &= ~(0x1 << 11);
            
    if (phy_flowctrl)	//FC on
    {
        //printf("<set FC on>");
        reg |= (0x1 << 11);
    }	
    else
    {
        //printf("<set FC off>");
        reg |= (0x0 << 11);        	            	
    }            
            
    reg &= ~(0x1 << 10);
            
    if ((phy_speed_dup & 0x2) || (phy_speed_dup & 0x8))	//FullDup
    {
        //printf("<set full>");
        reg |= (0x1 << 10);
    }
    else	//HalfDup
    {
        //printf("<set half>");
        reg |= (0x0 << 10); //Half          	            	                
    }
            
    eth_outl(GSW_PORT1_CFG_REG, reg);  


    /*
     * Check MAC port 1 link status
     */
    for (ii = 0; ii < 0x1000; ii++)
    {
    	reg = eth_inl(GSW_PORT1_CFG_REG);
    	
        if ((reg & 0x1) && !(reg & 0x2))
        {
            /*
             * enable MAC port 1
             */
            reg &= ~(0x1 << 18);

            /*
             * enable the forwarding of unknown, multicast and broadcast packets to CPU
             */
            reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));
        
            /*
             * include unknown, multicast and broadcast packets into broadcast storm
             */
            reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));
            
            eth_outl(GSW_PORT1_CFG_REG, reg);
            
            return;
        }
    }


    if (ii > 0x1000)
    {
        /*
         * Port 1 PHY link down or no TXC in Port 1
         */
        printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
        
        return;
    }
}

/******************************************************************************
 *
 * FUNCTION:  orn_configure_rtl8201cl_phy
 * PURPOSE:
 *
 *****************************************************************************/
#ifdef CONFIG_RTL8201CL
void orn_configure_rtl8201cl_phy(void)
{
    u16			phy_data = 0, phy_data2 = 0;
    u32 volatile	ii, jj;
    u8			phy_speed_dup = 0, phy_flowctrl = 0;
    u32 volatile	reg;


 /*
     * Configure MAC port 0
     */
    reg = eth_inl(GSW_PORT0_CFG_REG);
    
    // disable MAC's AN
    reg &= ~(0x1 << 7);
    eth_outl(GSW_PORT0_CFG_REG, reg);

    /* enable flow control on (PAUSE frame) */
    orn_phy_read(gsw_mac_0_phy_addr, 0x4, &phy_data);
    phy_data |= (0x1 << 10);  		
    orn_phy_write(gsw_mac_0_phy_addr, 0x4, phy_data);

    /* restart PHY auto neg. */
    orn_phy_read(gsw_mac_0_phy_addr, 0x0, &phy_data);
    phy_data |= (0x1 << 9) | (0x1 << 12);		
    orn_phy_write(gsw_mac_0_phy_addr, 0x0, phy_data);

    /* wait for PHY auto neg. complete */
    udelay(1500);//alan		
    for (ii = 0; ii < 0x20; ii++)
    {
        orn_phy_read(gsw_mac_0_phy_addr, 0x1, &phy_data);
	if(phy_data & 0x4)
        {
            break;
        }
        else
        {			
            if ((ii % 4) == 0)
                printf("\rCheck MAC/PHY 0 Link Status : |");
            else if ((ii % 4) == 1)
                printf("\rCheck MAC/PHY 0 Link Status : /");
            else if ((ii % 4) == 2)
                printf("\rCheck MAC/PHY 0 Link Status : -");
            else if ((ii % 4) == 3)
                printf("\rCheck MAC/PHY 0 Link Status : \\");
        }
    }		

	
    if (ii >= 0x20)
    {
        printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
       // return;
    }
    else
    {
        //printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
	orn_phy_read(gsw_mac_0_phy_addr, 0x4, &phy_data);
    	orn_phy_read(gsw_mac_0_phy_addr, 0x5, &phy_data2);
    	if (phy_data & 0x0400)	//FC on
    	{
        	//printf("<FC ON>");
        	phy_flowctrl = 1;
    	}
    	else    
    	{
        	// printf("<FC OFF>");
        	phy_flowctrl = 0;
    	}    
       
    	phy_speed_dup = 0;
    
   	 if ((phy_data & 0x0100) && (phy_data2 & 0x0100)) //100F
    	{
        	// printf("<100F>");
        	phy_speed_dup |= (0x1 << 3); //set bit3 for 100F
    	}
    	else if ((phy_data & 0x0080) && (phy_data2 & 0x0080)) //100F
    	{
        	// printf("<100H>");
        	phy_speed_dup |= (0x1 << 2);
    	}
    	else if ((phy_data & 0x0040) && (phy_data2 & 0x0040)) //100F
    	{
        	// printf("<10F>");
        	phy_speed_dup |= (0x1 << 1);
    	}
    	else if ((phy_data & 0x0020) && (phy_data2 & 0x0020)) //100F
    	{
        	// printf("<10H>");
        	phy_speed_dup |= 0x1;    
    	}
	    /*
     	* Configure MAC port 0 in forced setting subject to the current PHY status
     */     
    	reg = eth_inl(GSW_PORT0_CFG_REG);
    	reg &= ~(0x1 << 7);	//AN off
    	reg &= ~(0x3 << 8);
          
    	if (phy_speed_dup & 0x0C)     //100
    	{          
        	//printf("<set 100>");
        	reg |= (0x01 << 8);              
    	}
    	else if (phy_speed_dup & 0x03)     //10          
    	{
        	//printf("<set 10>");
        	reg |= (0x00 << 8);
    	}
        reg &= ~(0x1 << 11);
            
    	if (phy_flowctrl)	//FC on
    	{
        	//printf("<set FC on>");
       	 	reg |= (0x1 << 11);
    	}	
    	else
    	{
        	//printf("<set FC off>");
        	reg |= (0x0 << 11);        	            	
    	}            
            
    	reg &= ~(0x1 << 10);
            
    	if ((phy_speed_dup & 0x2) || (phy_speed_dup & 0x8))	//FullDup
    	{
        	//printf("<set full>");
        	reg |= (0x1 << 10);
    	}
    	else	//HalfDup
    	{
        	//printf("<set half>");
        	reg |= (0x0 << 10); //Half          	            	                
    	}
            
    	eth_outl(GSW_PORT0_CFG_REG, reg);  
    	/*
     	* Check MAC port 0 link status
     	*/
    	for (ii = 0; ii < 0x1000; ii++)
    	{
    		reg = eth_inl(GSW_PORT0_CFG_REG);
        	if ((reg & 0x1) && !(reg & 0x2))
        	{
            	/*
             	* enable MAC port 0
             	*/
            	reg &= ~(0x1 << 18);
            	/*
             	* enable the forwarding of unknown, multicast and broadcast packets to CPU
             */
            	reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));
            	/*
            	 * include unknown, multicast and broadcast packets into broadcast storm
             */
            	reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));
            	eth_outl(GSW_PORT0_CFG_REG, reg);
		break;
               //        return;
        	}
    	}

    	if (ii >= 0x1000)
    	{
        	/*
        	 * Port 0 PHY link down or no TXC in Port 1
         	*/
        	printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
		
             // return;
    	}
	else
		printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
    }


    
	
       /*
     * Configure MAC port 1
     */
    reg = eth_inl(GSW_PORT1_CFG_REG);
    
    // disable MAC's AN
    reg &= ~(0x1 << 7);
    eth_outl(GSW_PORT1_CFG_REG, reg);

    /* enable flow control on (PAUSE frame) */
    orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_data);
    phy_data |= (0x1 << 10);  		
    orn_phy_write(gsw_mac_1_phy_addr, 0x4, phy_data);

    /* restart PHY auto neg. */
    orn_phy_read(gsw_mac_1_phy_addr, 0x0, &phy_data);
    phy_data |= (0x1 << 9) | (0x1 << 12);		
    orn_phy_write(gsw_mac_1_phy_addr, 0x0, phy_data);

    /* wait for PHY auto neg. complete */
    udelay(1500);//alan		
    for (ii = 0; ii < 0x20; ii++)
    {
        orn_phy_read(gsw_mac_1_phy_addr, 0x1, &phy_data);
 	if(phy_data & 0x4)
        {
            break;
        }
        else
        {			
            if ((ii % 4) == 0)
                printf("\rCheck MAC/PHY 1 Link Status : |");
            else if ((ii % 4) == 1)
                printf("\rCheck MAC/PHY 1 Link Status : /");
            else if ((ii % 4) == 2)
                printf("\rCheck MAC/PHY 1 Link Status : -");
            else if ((ii % 4) == 3)
                printf("\rCheck MAC/PHY 1 Link Status : \\");
        }
    }		

	
    if (ii >= 0x20)
    {
        printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
        //return;
    }
    else
    {
     	orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_data);
    	orn_phy_read(gsw_mac_1_phy_addr, 0x5, &phy_data2);
    	if (phy_data & 0x0400)	//FC on
    	{
        	//printf("<FC ON>");
        	phy_flowctrl = 1;
    	}
    	else    
    	{
        	// printf("<FC OFF>");
       	 	phy_flowctrl = 0;
    	}    
       
    	phy_speed_dup = 0;
    	if ((phy_data & 0x0100) && (phy_data2 & 0x0100)) //100F
    	{
        	// printf("<100F>");
        	phy_speed_dup |= (0x1 << 3); //set bit3 for 100F
    	}
    	else if ((phy_data & 0x0080) && (phy_data2 & 0x0080)) //100F
    	{
        	// printf("<100H>");
        	phy_speed_dup |= (0x1 << 2);
    	}
    	else if ((phy_data & 0x0040) && (phy_data2 & 0x0040)) //100F
    	{
        	// printf("<10F>");
        	phy_speed_dup |= (0x1 << 1);
    	}
   	else if ((phy_data & 0x0020) && (phy_data2 & 0x0020)) //100F
    	{
        	// printf("<10H>");
        	phy_speed_dup |= 0x1;    
    	}
 	   /*
     	* Configure MAC port 1 in forced setting subject to the current PHY status
     */     
    	reg = eth_inl(GSW_PORT1_CFG_REG);
        reg &= ~(0x1 << 7);	//AN off
        reg &= ~(0x3 << 8);
            
    	if (phy_speed_dup & 0x0C)     //100
    	{          
        	//printf("<set 100>");
        	reg |= (0x01 << 8);              
    	}
    	else if (phy_speed_dup & 0x03)     //10          
    	{
        	//printf("<set 10>");
        	reg |= (0x00 << 8);
    	}
        reg &= ~(0x1 << 11);
            
    	if (phy_flowctrl)	//FC on
    	{
        	//printf("<set FC on>");
        	reg |= (0x1 << 11);
    	}	
    	else
    	{
        	//printf("<set FC off>");
        	reg |= (0x0 << 11);        	            	
    	}            
        reg &= ~(0x1 << 10);
            
    	if ((phy_speed_dup & 0x2) || (phy_speed_dup & 0x8))	//FullDup
    	{
        	//printf("<set full>");
        	reg |= (0x1 << 10);
    	}
    	else	//HalfDup
    	{
        	//printf("<set half>");
        	reg |= (0x0 << 10); //Half          	            	                
    	}
            
    	eth_outl(GSW_PORT1_CFG_REG, reg);  
	    /*
     	* Check MAC port 1 link status
     	*/
    	for (ii = 0; ii < 0x1000; ii++)
    	{
    		reg = eth_inl(GSW_PORT1_CFG_REG);
    	        if ((reg & 0x1) && !(reg & 0x2))
        	{
            	/*
             	* enable MAC port 1
             	*/
            	reg &= ~(0x1 << 18);
	            /*
        	     * enable the forwarding of unknown, multicast and broadcast packets to CPU
           	  	*/
            	reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));
               	/*
             	* include unknown, multicast and broadcast packets into broadcast storm
             */
            	reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));
               	eth_outl(GSW_PORT1_CFG_REG, reg);
            //    return;
		break;	
      		  }	
    	}

    	if (ii >= 0x1000)
    	{
        	/*
         	* Port 1 PHY link down or no TXC in Port 1
         	*/
      		printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
        }
	else
		printf("\rCheck MAC/PHY 1 Link Status : UP!\n");   
    }


    
}
#endif
#ifdef CONFIG_DM9161AE
void orn_configure_dm9161ae_phy(void)
{
    u16			phy_data = 0, phy_data2 = 0;
    u32 volatile	ii, jj;
    u8			phy_speed_dup = 0, phy_flowctrl = 0;
    u32 volatile	reg;


 /*
     * Configure MAC port 0
     */
    reg = eth_inl(GSW_PORT0_CFG_REG);
    
    // disable MAC's AN
    reg &= ~(0x1 << 7);
    eth_outl(GSW_PORT0_CFG_REG, reg);

    /* enable flow control on (PAUSE frame) */
    orn_phy_read(gsw_mac_0_phy_addr, 0x4, &phy_data);
    phy_data |= (0x1 << 10);  		
    orn_phy_write(gsw_mac_0_phy_addr, 0x4, phy_data);

    /* restart PHY auto neg. */
    orn_phy_read(gsw_mac_0_phy_addr, 0x0, &phy_data);
    phy_data = (0x1 << 9) | (0x1 << 12);//|=	
    orn_phy_write(gsw_mac_0_phy_addr, 0x0, phy_data);

    /* wait for PHY auto neg. complete */
    udelay(2500);//alan		
    for (ii = 0; ii < 0x20; ii++)
    {
        orn_phy_read(gsw_mac_0_phy_addr, 0x1, &phy_data);
	//printf("reg1:0x%x\n", phy_data);
	if(phy_data & 0x4) /* Link Status */
        {
            break;
        }
        else
        {			
            if ((ii % 4) == 0)
                printf("\rCheck MAC/PHY 0 Link Status : |");
            else if ((ii % 4) == 1)
                printf("\rCheck MAC/PHY 0 Link Status : /");
            else if ((ii % 4) == 2)
                printf("\rCheck MAC/PHY 0 Link Status : -");
            else if ((ii % 4) == 3)
                printf("\rCheck MAC/PHY 0 Link Status : \\");
        }
    }		

	
    if (ii >= 0x20)
    {
        //printf("\ralan-Check MAC/PHY 0 Link Status : DOWN!\n");
       // return;
    }
    else
    {
       // printf("\ralan-Check MAC/PHY 0 Link Status : UP!\n");
	orn_phy_read(gsw_mac_0_phy_addr, 0x4, &phy_data);
    	orn_phy_read(gsw_mac_0_phy_addr, 0x5, &phy_data2);
    	if (phy_data & 0x0400)	//FC on
    	{
        	//printf("<FC ON>");
        	phy_flowctrl = 1;
    	}
    	else    
    	{
        	// printf("<FC OFF>");
        	phy_flowctrl = 0;
    	}    
       
    	phy_speed_dup = 0;
    
   	 if ((phy_data & 0x0100) && (phy_data2 & 0x0100)) //100F
    	{
        	// printf("<100F>");
        	phy_speed_dup |= (0x1 << 3); //set bit3 for 100F
    	}
    	else if ((phy_data & 0x0080) && (phy_data2 & 0x0080)) //100F
    	{
        	// printf("<100H>");
        	phy_speed_dup |= (0x1 << 2);
    	}
    	else if ((phy_data & 0x0040) && (phy_data2 & 0x0040)) //100F
    	{
        	// printf("<10F>");
        	phy_speed_dup |= (0x1 << 1);
    	}
    	else if ((phy_data & 0x0020) && (phy_data2 & 0x0020)) //100F
    	{
        	// printf("<10H>");
        	phy_speed_dup |= 0x1;    
    	}
	    /*
     	* Configure MAC port 0 in forced setting subject to the current PHY status
     */     
    	reg = eth_inl(GSW_PORT0_CFG_REG);
    	reg &= ~(0x1 << 7);	//AN off
    	reg &= ~(0x3 << 8);
          
    	if (phy_speed_dup & 0x0C)     //100
    	{          
        	//printf("<set 100>");
        	reg |= (0x01 << 8);              
    	}
    	else if (phy_speed_dup & 0x03)     //10          
    	{
        	//printf("<set 10>");
        	reg |= (0x00 << 8);
    	}
        reg &= ~(0x1 << 11);
            
    	if (phy_flowctrl)	//FC on
    	{
        	//printf("<set FC on>");
       	 	reg |= (0x1 << 11);
    	}	
    	else
    	{
        	//printf("<set FC off>");
        	reg |= (0x0 << 11);        	            	
    	}            
            
    	reg &= ~(0x1 << 10);
            
    	if ((phy_speed_dup & 0x2) || (phy_speed_dup & 0x8))	//FullDup
    	{
        	//printf("<set full>");
        	reg |= (0x1 << 10);
    	}
    	else	//HalfDup
    	{
        	//printf("<set half>");
        	reg |= (0x0 << 10); //Half          	            	                
    	}
            
    	eth_outl(GSW_PORT0_CFG_REG, reg);  
    	/*
     	* Check MAC port 0 link status
     	*/
    	for (ii = 0; ii < 0x1000; ii++)
    	{
    		reg = eth_inl(GSW_PORT0_CFG_REG);
        	if ((reg & 0x1) && !(reg & 0x2))
        	{
            	/*
             	* enable MAC port 0
             	*/
            	reg &= ~(0x1 << 18);
            	/*
             	* enable the forwarding of unknown, multicast and broadcast packets to CPU
             */
            	reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));
            	/*
            	 * include unknown, multicast and broadcast packets into broadcast storm
             */
            	reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));
            	eth_outl(GSW_PORT0_CFG_REG, reg);
		break;
               //        return;
        	}
    	}

    	if (ii >= 0x1000)
    	{
        	/*
        	 * Port 0 PHY link down or no TXC in Port 1
         	*/
        	printf("\rCheck MAC/PHY 0 Link Status : DOWN!\n");
		
             // return;
    	}
	else
		printf("\rCheck MAC/PHY 0 Link Status : UP!\n");
    }


    
	
       /*
     * Configure MAC port 1
     */
    reg = eth_inl(GSW_PORT1_CFG_REG);
    
    // disable MAC's AN
    reg &= ~(0x1 << 7);
    eth_outl(GSW_PORT1_CFG_REG, reg);

    /* enable flow control on (PAUSE frame) */
    orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_data);
    phy_data |= (0x1 << 10);  		
    orn_phy_write(gsw_mac_1_phy_addr, 0x4, phy_data);

    /* restart PHY auto neg. */
    orn_phy_read(gsw_mac_1_phy_addr, 0x0, &phy_data);
    phy_data |= (0x1 << 9) | (0x1 << 12);		
    orn_phy_write(gsw_mac_1_phy_addr, 0x0, phy_data);

    /* wait for PHY auto neg. complete */
    udelay(1500);//alan		
    for (ii = 0; ii < 0x20; ii++)
    {
        orn_phy_read(gsw_mac_1_phy_addr, 0x1, &phy_data);
 	if(phy_data & 0x4)
        {
            break;
        }
        else
        {			
            if ((ii % 4) == 0)
                printf("\rCheck MAC/PHY 1 Link Status : |");
            else if ((ii % 4) == 1)
                printf("\rCheck MAC/PHY 1 Link Status : /");
            else if ((ii % 4) == 2)
                printf("\rCheck MAC/PHY 1 Link Status : -");
            else if ((ii % 4) == 3)
                printf("\rCheck MAC/PHY 1 Link Status : \\");
        }
    }		

	
    if (ii >= 0x20)
    {
        printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
        //return;
    }
    else
    {
     	orn_phy_read(gsw_mac_1_phy_addr, 0x4, &phy_data);
    	orn_phy_read(gsw_mac_1_phy_addr, 0x5, &phy_data2);
    	if (phy_data & 0x0400)	//FC on
    	{
        	//printf("<FC ON>");
        	phy_flowctrl = 1;
    	}
    	else    
    	{
        	// printf("<FC OFF>");
       	 	phy_flowctrl = 0;
    	}    
       
    	phy_speed_dup = 0;
    	if ((phy_data & 0x0100) && (phy_data2 & 0x0100)) //100F
    	{
        	// printf("<100F>");
        	phy_speed_dup |= (0x1 << 3); //set bit3 for 100F
    	}
    	else if ((phy_data & 0x0080) && (phy_data2 & 0x0080)) //100F
    	{
        	// printf("<100H>");
        	phy_speed_dup |= (0x1 << 2);
    	}
    	else if ((phy_data & 0x0040) && (phy_data2 & 0x0040)) //100F
    	{
        	// printf("<10F>");
        	phy_speed_dup |= (0x1 << 1);
    	}
   	else if ((phy_data & 0x0020) && (phy_data2 & 0x0020)) //100F
    	{
        	// printf("<10H>");
        	phy_speed_dup |= 0x1;    
    	}
 	   /*
     	* Configure MAC port 1 in forced setting subject to the current PHY status
     */     
    	reg = eth_inl(GSW_PORT1_CFG_REG);
        reg &= ~(0x1 << 7);	//AN off
        reg &= ~(0x3 << 8);
            
    	if (phy_speed_dup & 0x0C)     //100
    	{          
        	//printf("<set 100>");
        	reg |= (0x01 << 8);              
    	}
    	else if (phy_speed_dup & 0x03)     //10          
    	{
        	//printf("<set 10>");
        	reg |= (0x00 << 8);
    	}
        reg &= ~(0x1 << 11);
            
    	if (phy_flowctrl)	//FC on
    	{
        	//printf("<set FC on>");
        	reg |= (0x1 << 11);
    	}	
    	else
    	{
        	//printf("<set FC off>");
        	reg |= (0x0 << 11);        	            	
    	}            
        reg &= ~(0x1 << 10);
            
    	if ((phy_speed_dup & 0x2) || (phy_speed_dup & 0x8))	//FullDup
    	{
        	//printf("<set full>");
        	reg |= (0x1 << 10);
    	}
    	else	//HalfDup
    	{
        	//printf("<set half>");
        	reg |= (0x0 << 10); //Half          	            	                
    	}
            
    	eth_outl(GSW_PORT1_CFG_REG, reg);  
	    /*
     	* Check MAC port 1 link status
     	*/
    	for (ii = 0; ii < 0x1000; ii++)
    	{
    		reg = eth_inl(GSW_PORT1_CFG_REG);
    	        if ((reg & 0x1) && !(reg & 0x2))
        	{
            	/*
             	* enable MAC port 1
             	*/
            	reg &= ~(0x1 << 18);
	            /*
        	     * enable the forwarding of unknown, multicast and broadcast packets to CPU
           	  	*/
            	reg &= ~((0x1 << 25) | (0x1 << 26) | (0x1 << 27));
               	/*
             	* include unknown, multicast and broadcast packets into broadcast storm
             */
            	reg |= ((0x1 << 29) | (0x1 << 30) | ((u_int32)0x1 << 31));
               	eth_outl(GSW_PORT1_CFG_REG, reg);
            //    return;
		break;	
      		  }	
    	}

    	if (ii >= 0x1000)
    	{
        	/*
         	* Port 1 PHY link down or no TXC in Port 1
         	*/
      		printf("\rCheck MAC/PHY 1 Link Status : DOWN!\n");
        }
	else
		printf("\rCheck MAC/PHY 1 Link Status : UP!\n");   
    }


    
}
#endif
static int orn_drv_close(orn_device *dev)
{
	u32 volatile    reg;
	
	
	DBG_FUNC(1);


	/* disable all interrupts */
	eth_outl(GSW_INT_MSK_REG, GSW_INT_MSK_ALL);
	
	/* clear all interrupts */
	eth_outl(GSW_INT_STAT_REG, GSW_INT_MSK_ALL);
	
	/* disable DMA Interrupts */
	reg = eth_inl(INT_MSK_REG);
	
	reg |= (INT_GSW_TS_TX_CMPLT | INT_GSW_FS_RX_CMPLT | INT_GSW_TS_Q_EMPTY | INT_GSW_FS_Q_FULL);
	
	eth_outl(INT_MSK_REG, reg);
	
	/* clear DMA Interrupts */
	reg = eth_inl(INT_CLR_REG);
	
	reg |= (INT_GSW_TS_TX_CMPLT | INT_GSW_FS_RX_CMPLT | INT_GSW_TS_Q_EMPTY | INT_GSW_FS_Q_FULL);
	
	eth_outl(INT_CLR_REG, reg);


	/*
	 * disable CPU port, GSW MAC port 0 and MAC port 1
	 */
        eth_outl(GSW_CPU_PORT_CFG_REG, eth_inl(GSW_CPU_PORT_CFG_REG) | (0x1 << 18));

        eth_outl(GSW_PORT0_CFG_REG, eth_inl(GSW_PORT0_CFG_REG) | (0x1 << 18));
       
        eth_outl(GSW_PORT1_CFG_REG, eth_inl(GSW_PORT1_CFG_REG) | (0x1 << 18));


	/*
	 * disable TS_DMA and FS_DMA
	 */
	eth_outl(GSW_TS_DMA_CTL_REG, GSW_DMA_DIS);
	
	eth_outl(GSW_FS_DMA_CTL_REG, GSW_DMA_DIS);


	/*
	 * reset GSW
	 */
	// set high
	reg = eth_inl(0x77000000 + 0x4);	
	reg |= (0x1 << 4);
	eth_outl(0x77000000 + 0x4, reg);
	
	// set low
	reg = eth_inl(0x77000000 + 0x4);	
	reg &= ~(0x1 << 4);
	eth_outl(0x77000000 + 0x4, reg);

	// set high
	reg = eth_inl(0x77000000 + 0x4);	
	reg |= (0x1 << 4);
	eth_outl(0x77000000 + 0x4, reg);


	// waiting until memory BIST is complete
	while (1)
	{
		if (eth_inl(0x70000000 + 0x94) & (0x1 << 17))
		{
			//printf("\nGSW Memory BIST complete!\n");
			
			break;
		}
	}


	if (dev)
	{
		if(dev->hw_tx_desc)
		{
			free(dev->hw_tx_desc);
		}		
		
		if(dev->hw_rx_desc)
		{
			free(dev->hw_rx_desc);
		}
		
		if(dev->drv_tx_desc)
		{
			free(dev->drv_tx_desc);
		}
		
		if(dev->drv_rx_desc)
		{
			free(dev->drv_rx_desc);
		}
		
		if(dev->pkt_pool)
		{
			free(dev->pkt_pool);
		}
		
		if(dev->buf_pool)
		{
			free(dev->buf_pool);
		}
	
		free(dev);
	}
	
	return 0;
}


static int orn_drv_rx(orn_device *dev)
{
	u32 loop_cnt = 0, idx;
	drv_pkt_desc *pkt;
	orn_rx_desc  *rx_desc;
	
	DBG_FUNC(1);
	
	idx = dev->rx_idx;
	rx_desc = &dev->hw_rx_desc[idx];
	
	while(rx_desc->stat_reg & DMA_CPU_OWN)
	{
		pkt = dev->drv_rx_desc[idx];
		
		pkt->flag = rx_desc->stat_reg;
		pkt->src = (pkt->flag & DMA_RX_SRC_PORT_MSK);
		pkt->len = (pkt->flag & DMA_RX_LEN_MSK);

		dev->drv_rx_desc[idx] = orn_get_free_pkt(dev);
		
		if(dev->drv_rx_desc[idx] == NULL)
		{
			/* if no free packet, drop this one */
			DBG_PRINT(0, "no free packets\n");
			dev->rx_buf_full = TRUE;					
			rx_desc->stat_reg &= DMA_END_OF_RING;
			rx_desc->stat_reg |= PKT_BUFF_SIZE;
			
			goto orn_packet_drop;
		}
			
		rx_desc->buf_reg = (u32)dev->drv_rx_desc[idx]->buf;

		/* reset state reg, also implies DMA_CPU_OWN will be clear */		
		rx_desc->stat_reg &= DMA_END_OF_RING;
		rx_desc->stat_reg |= PKT_BUFF_SIZE;

		DBG_PRINT(2, "receive packet context\n");
		orn_print_packet(pkt->buf, pkt->len);
		
		NetReceive(pkt->buf, pkt->len);
		
		/* return packet to free packet list */
		orn_save_free_pkt(dev, pkt);		
		
orn_packet_drop:
		
		if(++idx >= dev->num_rx_desc)
		{
			idx = 0;
		}
		
		eth_outl(GSW_FS_DMA_CTL_REG, GSW_DMA_EN);
		
		if(++loop_cnt > dev->num_rx_desc)
		{
			break;
		}
		
		rx_desc = &dev->hw_rx_desc[idx];		
	}
	
	dev->rx_idx = idx;
	
	dev->cnt_rx_pkt += loop_cnt;
	
	return 0;
}



static int orn_drv_send(orn_device *dev)
{
	orn_tx_desc	*tx_desc;
	drv_pkt_desc	*pkt = NULL;
	u16		cur_idx, next_idx, tx_cnt = 0;


	DBG_FUNC(1);
	
	ASSERT(dev);
	
	/* dequeue from tx pkt queue */
	pkt = orn_dequeue(&(dev->tx_pkt_queue));
	
	if(!pkt)
	{
		return 0;
	}


	DBG_PRINT(2, "send packet context\n");
	
	orn_print_packet(pkt->buf, pkt->len);

	cur_idx = next_idx = dev->tx_idx_head;
	
	while(pkt != NULL)
	{
		if(++next_idx == dev->num_tx_desc)
		{
			next_idx = 0;
		}
		
		if(next_idx == dev->tx_idx_tail)
		{
		    DBG_PRINT(0,"tx buffer full\n");
		    
		    break;
		}
		
		tx_desc = &dev->hw_tx_desc[cur_idx];
		
		if(!(tx_desc->ctl_reg & DMA_CPU_OWN))
		{
			break;
		}
		
		dev->drv_tx_desc[cur_idx] = pkt;

		pkt->dst = (DMA_TX_GE_PORT_0 | DMA_TX_GE_PORT_1);
		
		tx_desc->buf_reg = (u32)pkt->buf;


		/*
		 * Note here we do not use forced port map transmission
		 */
   		tx_desc->ctl_reg |= ((pkt->len & DMA_TX_LEN_MSK) |
   				     pkt->dst |
   				     DMA_TX_INT_EN |
   				     DMA_LST_SEG |
   				     DMA_FST_SEG);

		
		tx_desc->ctl_reg &= ~DMA_CPU_OWN;


		if (orn_icplus175c_mii_mode)
		{
			/*
			 * ICPlus175C is in single MII mode			
			 * Instruct TX_DMA to inset VLAN GID 1's VLAN tag
			 */
			tx_desc->hdr_reg &= ~(0xF);
			
			tx_desc->hdr_reg |= (0x1 << 3) | (0x1);
		}
			                  
			                   
		cur_idx = next_idx;
	  
		if(++tx_cnt > dev->num_tx_desc)
		{
	  		pkt = NULL;
	  	}
		else
		{
			pkt = orn_dequeue(&(orn_dev->tx_pkt_queue));
		}
	}
	
	dev->tx_idx_head = cur_idx;
	
	/* trigger DMA to send */
	eth_outl(GSW_TS_DMA_CTL_REG, GSW_DMA_EN);
	
	/* queue pkt if buffer full */
	if(pkt != NULL)
	{
		orn_enqueue_head(&(orn_dev->tx_pkt_queue), pkt);
	}
	
	/* wait for packet transmit complete */
	DBG_PRINT(2, "wait for packet transmit...\n");
	
	while(!(eth_inl(INT_SRC_REG) & (INT_GSW_TS_TX_CMPLT | INT_GSW_TS_Q_EMPTY)))
	{
		/* just wait */
	}
	
	orn_drv_send_complete(dev);
	
	/* clear interrupt */
	eth_outl(GSW_INT_STAT_REG, eth_inl(GSW_INT_STAT_REG));
	eth_outl(INT_CLR_REG, (INT_GSW_TS_TX_CMPLT | INT_GSW_TS_Q_EMPTY));
	
	return 0;
}


static void orn_drv_send_complete(orn_device *dev)
{
	u32 loop_cnt = 0, idx;
	drv_pkt_desc *pkt;
	
	DBG_FUNC(2);
	
	idx = dev->tx_idx_tail;
	
	while((dev->hw_tx_desc[idx].ctl_reg & DMA_CPU_OWN) && (idx != dev->tx_idx_head))
	{
		pkt = dev->drv_tx_desc[idx];
		
		dev->drv_tx_desc[idx] = NULL;
		
		/* clear hw desc except DMA_END_OF_RING */
		dev->hw_tx_desc[idx].buf_reg = (u32)NULL;
		dev->hw_tx_desc[idx].ctl_reg &= DMA_END_OF_RING;
		dev->hw_tx_desc[idx].ctl_reg |= DMA_CPU_OWN;
		
		/* return packet to free packet list */
		orn_save_free_pkt(dev, pkt);
		
		if(++idx == dev->num_tx_desc)
		{
			idx = 0;
		}
		
		if(++loop_cnt > dev->num_tx_desc)
		{
			break;
		}		
	}
	
	dev->tx_idx_tail = idx;
	
	dev->cnt_tx_pkt += loop_cnt;
	
	/* try to send packet in tx queue */
	orn_drv_send(dev);	
}


static drv_pkt_desc *orn_get_free_pkt(orn_device *dev)
{
	drv_pkt_desc *pkt = NULL;
	
		
	DBG_FUNC(3);
	
	ASSERT(dev);
	ASSERT(dev->free_pkt_list);
	
	pkt = dev->free_pkt_list;
	dev->free_pkt_list = pkt->next;
	pkt->next = NULL;
	
	return pkt;
	
}


static int orn_save_free_pkt(orn_device *dev, drv_pkt_desc *pkt)
{
	
	DBG_FUNC(3);
	
	ASSERT(dev);
	ASSERT(dev->free_pkt_list);
	ASSERT(pkt);
	
	pkt->next = dev->free_pkt_list;
	dev->free_pkt_list = pkt;
	
	return 0;
}


static void * orn_drv_alloc_mem(u32 bytes, u8 clear)
{
	void *mem;
	u32 align_mem;
	
	DBG_FUNC(3);
	
	mem = malloc(bytes + 0x3);
	
	if(!mem)
	{
		
		return (void *)NULL;
	}
	
  	align_mem = (u32)mem;
  
  	if(align_mem & 0x3)
  	{
		align_mem = (align_mem + 0x3) & ~0x3;
	}
  
	mem = (void *)align_mem;
  
  	if(clear)
  	{
		memset((void *)mem, 0, bytes);
	}
     
	return mem;  
}


#endif	/* CONFIG_STR9100_GSW */
