#ifndef _STAR_CPU_H_
#define _STAR_CPU_H_

/******************************************************************************
 * MODULE NAME:    star_cpu.h
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           07 July 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    07 July 2003  -  MJLIU	- Initial Version v1.0
 *
 *
 * SOURCE:
 * ISSUES:
 * NOTES TO USERS:
 ******************************************************************************/

#include "star_mmu_h.h"


/* 
 * Bit definitions for the control register
 */

#define EnableMMU                       0x0001
#define EnableAlignFault                0x0002
#define EnableDcache                    0x0004
#define EnableWB                        0x0008
#define EnableBigEndian                 0x0080
#define EnableMMU_S                     0x0100        /* selects MMU access checks */
#define EnableMMU_R                     0x0200        /* selects MMU access checks */
#define EnableBTB                       0x0800
#define EnableIcache                    0x1000


#define BranchPrediction                0x0800
#define EnableHighVectors               0x2000        /* Vectors relocated to 0xFFFF0000 */


/*  
 * Definitions used in conditional assembly of Icache, Dcache and Write Buffer
 * options
 */
#define IC_ON                           EnableIcache
#define IC_OFF                          0x0

#define DC_ON                           EnableDcache
#define DC_OFF                          0x0

#define WB_ON                           EnableWB
#define WB_OFF                          0x0

#define BTB_ON                          EnableBTB
#define BTB_OFF                         0x0

#define BIGENDIAN_ON                    EnableBigEndian
#define BIGENDIAN_OFF                   0x0


/*
 *  Function Declaration
 */
int     Hal_Cpu_Read_MMU_Configuration(void);
void    Hal_Cpu_Write_MMU_Configuration(int);

int     Hal_Cpu_Read_Domain_Access_Control(void);
void    Hal_Cpu_Write_Domain_Access_Control(int);

void    Hal_Cpu_Enable_ICache(void);
void    Hal_Cpu_Invalidate_ICache_All(void);
void    Hal_Cpu_Invalidate_ICache_Entry(int);
void    Hal_Cpu_Prefetch_ICache_Entry(int);
void    Hal_Cpu_Prefetch_ICache_Flush(void);
void    Hal_Cpu_Enable_ICache_Lockdown(void);
void    Hal_Cpu_Disable_ICache_Lockdown(void);
void    Hal_Cpu_Lockdown_ICache_Entry(int);

void    Hal_Cpu_Enable_DCache(void);
void    Hal_Cpu_Invalidate_DCache_All(void);
void    Hal_Cpu_Invalidate_DCache_Entry(int);
void    Hal_Cpu_Invalidate_DCache_Range(int, int);
void    Hal_Cpu_Clean_DCache_All(void);
void    Hal_Cpu_Clean_DCache_Entry(int);
void    Hal_Cpu_Clean_DCache_Range(int, int);
void    Hal_Cpu_Clean_Invalidate_DCache_All(void);
void    Hal_Cpu_Clean_Invalidate_DCache_Entry(int);
void    Hal_Cpu_Clean_Invalidate_DCache_Range(int, int);
void    Hal_Cpu_Enable_DCache_Lockdown(void);
void    Hal_Cpu_Disable_DCache_Lockdown(void);
void    Hal_Cpu_Lockdown_DCache_Entry(int);

void    Hal_Cpu_Invalidate_ICache_DCache_All(void);

void    Hal_Cpu_Enable_Write_Buffer(void);
void    Hal_Cpu_Drain_Write_Buffer(void);

void    Hal_Cpu_Invalidate_UTLB_All(void);
void    Hal_Cpu_Invalidate_UTLB_Entry(int);
void    Hal_Cpu_Invalidate_ITLB_All(void);
void    Hal_Cpu_Invalidate_ITLB_Entry(int);
void    Hal_Cpu_Invalidate_DTLB_All(void);
void    Hal_Cpu_Invalidate_DTLB_Entry(int);

int     Hal_Cpu_Check_ECR_State(void);
void    Hal_Cpu_Enable_ECR(void);
void    Hal_Cpu_Disable_ECR(void);

void    Hal_Cpu_Enable_Big_Endian(void);

void    Hal_Cpu_Enable_BTB_All(void);
void    Hal_Cpu_Invalidate_BTB_All(void);

void    Hal_Cpu_Invalidate_IScratchpad_All(void);
int     Hal_Cpu_Read_IScratchpad_Configuration(void);
void    Hal_Cpu_Write_IScratchpad_Configuration(int);
void    Hal_Cpu_Disable_IScratchpad(void);
void    Hal_Cpu_Enable_IScratchpad(int, int);

int     Hal_Cpu_Read_DScratchpad_Configuration(void);
void    Hal_Cpu_Write_DScratchpad_Configuration(int);
void    Hal_Cpu_Disable_DScratchpad(void);
void    Hal_Cpu_Enable_DScratchpad(int, int, int);

int     Hal_Cpu_Read_PID(void);
void    Hal_Cpu_Write_PID(int pid);

void    Hal_Cpu_Enter_Idle_Mode(void);
void    Hal_Cpu_Enter_Sleep_Mode(void);


#endif

