#ifndef _STAR_GPIO_H_
#define _STAR_GPIO_H_

/******************************************************************************
 * MODULE NAME:    star_gpio.h
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           30 June 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    30 June 2003  -  MJLIU	- Initial Version v1.0
 *
 *
 * SOURCE:
 * ISSUES:
 * NOTES TO USERS:
 ******************************************************************************/

#include "sys_apb_memory_map.h"


#define GPIO_BASE_ADDR                     APB_DEVICES_MEM_MAP_ADDR(GPIO_BASE)

#define GPIO_GPIO_DATA_OUT_REG_OFFSET      0x00
#define GPIO_GPIO_DATA_IN_REG_OFFSET       0x04
#define GPIO_PIN_DIR_REG_OFFSET            0x08
#define GPIO_PIN_BYPASS_REG_OFFSET         0x0C
#define GPIO_GPIO_DATA_SET_REG_OFFSET      0x10
#define GPIO_GPIO_DATA_CLEAR_REG_OFFSET    0x14

#define GPIO_INTR_ENABLE_REG_OFFSET        0x20
#define GPIO_INTR_RAW_STATE_REG_OFFSET     0x24
#define GPIO_INTR_MASKED_STATE_REG_OFFSET  0x28
#define GPIO_INTR_MASK_REG_OFFSET          0x2C
#define GPIO_INTR_CLEAR_REG_OFFSET         0x30
#define GPIO_INTR_TRIGGER_REG_OFFSET       0x34
#define GPIO_INTR_BOTH_REG_OFFSET          0x38
#define GPIO_INTR_RISE_NEG_REG_OFFSET      0x3C
#define GPIO_BOUNCE_ENABLE_REG_OFFSET      0x40
#define GPIO_BOUNCE_PRESCALE_REG_OFFSET    0x44

#define GPIO_FEATURE_REG_OFFSET            0x78
#define GPIO_REVISION_REG_OFFSET           0x7C



#define GPIO_MEM_MAP_ADDR(regname)         (GPIO_BASE_ADDR + GPIO_##regname##_OFFSET)

#define GPIO_MEM_MAP_VALUE(regname)        (*((u_int32 volatile *)GPIO_MEM_MAP_ADDR(regname)))



/*
 * define access macros
 */
#define GPIO_DATA_OUT             GPIO_MEM_MAP_VALUE(GPIO_DATA_OUT_REG)
#define GPIO_DATA_IN              GPIO_MEM_MAP_VALUE(GPIO_DATA_IN_REG)
#define GPIO_PIN_DIR              GPIO_MEM_MAP_VALUE(PIN_DIR_REG)

#define GPIO_PIN_BYPASS           GPIO_MEM_MAP_VALUE(PIN_BYPASS_REG)

#define GPIO_DATA_SET             GPIO_MEM_MAP_VALUE(GPIO_DATA_SET_REG)
#define GPIO_DATA_CLEAR           GPIO_MEM_MAP_VALUE(GPIO_DATA_CLEAR_REG)

#define GPIO_INTR_ENABLE          GPIO_MEM_MAP_VALUE(INTR_ENABLE_REG)
#define GPIO_INTR_RAW_STATE       GPIO_MEM_MAP_VALUE(INTR_RAW_STATE_REG)
#define GPIO_INTR_MASKED_STATE    GPIO_MEM_MAP_VALUE(INTR_MASKED_STATE_REG)
#define GPIO_INTR_MASK            GPIO_MEM_MAP_VALUE(INTR_MASK_REG)
#define GPIO_INTR_CLEAR           GPIO_MEM_MAP_VALUE(INTR_CLEAR_REG)

#define GPIO_INTR_TRIGGER         GPIO_MEM_MAP_VALUE(INTR_TRIGGER_REG)
#define GPIO_INTR_BOTH            GPIO_MEM_MAP_VALUE(INTR_BOTH_REG)
#define GPIO_INTR_RISE_NEG        GPIO_MEM_MAP_VALUE(INTR_RISE_NEG_REG)

#define GPIO_BOUNCE_ENABLE        GPIO_MEM_MAP_VALUE(BOUNCE_ENABLE_REG)
#define GPIO_BOUNCE_PRESCALE      GPIO_MEM_MAP_VALUE(BOUNCE_PRESCALE_REG)


/*
 * define constant macros
 */
#define GPIO_0_MASK             (1 << 0)
#define GPIO_1_MASK             (1 << 1)
#define GPIO_2_MASK             (1 << 2)
#define GPIO_3_MASK             (1 << 3)
#define GPIO_4_MASK             (1 << 4)
#define GPIO_5_MASK             (1 << 5)
#define GPIO_6_MASK             (1 << 6)
#define GPIO_7_MASK             (1 << 7)
#define GPIO_8_MASK             (1 << 8)
#define GPIO_9_MASK             (1 << 9)
#define GPIO_10_MASK            (1 << 10)
#define GPIO_11_MASK            (1 << 11)
#define GPIO_12_MASK            (1 << 12)
#define GPIO_13_MASK            (1 << 13)
#define GPIO_14_MASK            (1 << 14)
#define GPIO_15_MASK            (1 << 15)
#define GPIO_16_MASK            (1 << 16)
#define GPIO_17_MASK            (1 << 17)
#define GPIO_18_MASK            (1 << 18)
#define GPIO_19_MASK            (1 << 19)
#define GPIO_20_MASK            (1 << 20)



/*
 * functions declarations
 */
void    Hal_Gpio_Dummy_function(void);


/*
 * macro declarations
 */
#define HAL_GPIO_READ_DATA_OUT_STATE(data_out_state)	((data_out_state) = (GPIO_DATA_OUT))


#define HAL_GPIO_READ_DATA_IN_STATE(data_in_state)	((data_in_state) = (GPIO_DATA_IN))


#define HAL_GPIO_SET_DIRECTION_OUTPUT(gpio_index)	((GPIO_PIN_DIR) |= (gpio_index))


#define HAL_GPIO_SET_DIRECTION_INPUT(gpio_index)	((GPIO_PIN_DIR) &= ~(gpio_index))


#define HAL_GPIO_ENABLE_BYPASS_MODE(gpio_index)		((GPIO_PIN_BYPASS) |= (gpio_index))


#define HAL_GPIO_DISABLE_BYPASS_MODE(gpio_index)	((GPIO_PIN_BYPASS) &= ~(gpio_index))


#define HAL_GPIO_SET_DATA_OUT_HIGH(gpio_index)		((GPIO_DATA_SET) |= (gpio_index))


#define HAL_GPIO_SET_DATA_OUT_LOW(gpio_index)		((GPIO_DATA_CLEAR) |= (gpio_index))


#define HAL_GPIO_ENABLE_INTERRUPT(gpio_index)		((GPIO_INTR_ENABLE) |= (gpio_index))

    
#define HAL_GPIO_DISABLE_INTERRUPT(gpio_index)		((GPIO_INTR_ENABLE) &= ~(gpio_index))


#define HAL_GPIO_READ_INTERRUPT_RAW_STATE(raw_state)	((raw_state) = (GPIO_INTR_RAW_STATE))

    
#define HAL_GPIO_READ_INTERRUPT_MASKED_STATE(masked_raw_state)	((masked_raw_state) = (GPIO_INTR_MASKED_STATE))


#define HAL_GPIO_DISABLE_INTERRUPT_MASK(gpio_index)		((GPIO_INTR_MASK) &= ~(gpio_index))


#define HAL_GPIO_ENABLE_INTERRUPT_MASK(gpio_index)		((GPIO_INTR_MASK) |= (gpio_index))


#define HAL_GPIO_CLEAR_INTERRUPT(gpio_index)			((GPIO_INTR_CLEAR) = (gpio_index))


#define HAL_GPIO_SET_INTERRUPT_EDGE_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) &= ~(gpio_index))

    
#define HAL_GPIO_SET_INTERRUPT_LEVEL_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) |= (gpio_index))


#define HAL_GPIO_SET_INTERRUPT_SINGLE_EDGE_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) &= ~(gpio_index)); ((GPIO_INTR_BOTH) &= ~(gpio_index));


#define HAL_GPIO_SET_INTERRUPT_BOTH_EDGE_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) &= ~(gpio_index)); ((GPIO_INTR_BOTH) |= (gpio_index));


#define HAL_GPIO_SET_INTERRUPT_SINGLE_RISING_EDGE_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) &= ~(gpio_index)); ((GPIO_INTR_BOTH) &= ~(gpio_index)); ((GPIO_INTR_RISE_NEG) &= ~(gpio_index));


#define HAL_GPIO_SET_INTERRUPT_SINGLE_FALLING_EDGE_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) &= ~(gpio_index)); ((GPIO_INTR_BOTH) &= ~(gpio_index)); ((GPIO_INTR_RISE_NEG) |= (gpio_index));


#define HAL_GPIO_SET_INTERRUPT_HIGH_LEVEL_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) |= (gpio_index)); ((GPIO_INTR_RISE_NEG) &= ~(gpio_index));


#define HAL_GPIO_SET_INTERRUPT_LOW_LEVEL_TRIGGER_MODE(gpio_index)	((GPIO_INTR_TRIGGER) |= (gpio_index)); ((GPIO_INTR_RISE_NEG) |= (gpio_index));


#define HAL_GPIO_ENABLE_BOUNCE(gpio_index)	((GPIO_BOUNCE_ENABLE) |= (gpio_index))


#define HAL_GPIO_DISABLE_BOUNCE(gpio_index)	((GPIO_BOUNCE_ENABLE) &= ~(gpio_index))


#define HAL_GPIO_READ_BOUNCE_PRESCALE_RATIO(prescale_ratio)	((prescale_ratio) = ((GPIO_BOUNCE_PRESCALE) & 0x0000FFFF))


#define HAL_GPIO_WRITE_BOUNCE_PRESCALE_RATIO(prescale_ratio)	((GPIO_BOUNCE_PRESCALE) = (prescale_ratio & 0x0000FFFF))


#endif  // end of _STAR_GPIO_H_