#ifndef _STAR_SMC_H_
#define _STAR_SMC_H_

/******************************************************************************
 * MODULE NAME:    star_smc.h
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           06 June 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    06 June 2003  -  MJLIU	- Initial Version v1.0
 *
 *
 * SOURCE:
 * ISSUES:
 * NOTES TO USERS:
 ******************************************************************************/

#include "sys_apb_memory_map.h"


#define SMC_BASE_ADDR                               SYS_MEM_MAP_ADDR(STATIC_MEMORY_CONTROLLER_BASE)

#define SMC_MEMORY_BANK0_CONFIGURE_REG_OFFSET       0x00
#define SMC_MEMORY_BANK0_TIMING_PARAM_REG_OFFSET    0x04
#define SMC_MEMORY_BANK1_CONFIGURE_REG_OFFSET       0x08
#define SMC_MEMORY_BANK1_TIMING_PARAM_REG_OFFSET    0x0C


#define SMC_MEM_MAP_ADDR(regname)           (SMC_BASE_ADDR + SMC_##regname##_OFFSET)
 
#define SMC_MEM_MAP_VALUE(regname)          (*((u_int32 volatile *)SMC_MEM_MAP_ADDR(regname)))


#define SMC_MEM_BANK0_CONFIG                SMC_MEM_MAP_VALUE(MEMORY_BANK0_CONFIGURE_REG)
#define SMC_MEM_BANK0_TIMING_PARAM          SMC_MEM_MAP_VALUE(MEMORY_BANK0_TIMING_PARAM_REG)
#define SMC_MEM_BANK1_CONFIG                SMC_MEM_MAP_VALUE(MEMORY_BANK1_CONFIGURE_REG)
#define SMC_MEM_BANK1_TIMING_PARAM          SMC_MEM_MAP_VALUE(MEMORY_BANK1_TIMING_PARAM_REG)



#define SMC_BANK_WRITE_PROTECT		(0x1 << 11)	/* bit 11 */

#define SMC_BANK_TYPE1_ASYNC		(0x0 << 10)	/* bit 10 */
#define SMC_BANK_TYPE1_SYNC		(0x1 << 10)

#define SMC_BANK_TYPE2_ASYNC		(0x0 << 9)	/* bit 9 */
#define SMC_BANK_TYPE2_BURST_ROM	(0x1 << 9)
#define SMC_BANK_TYPE2_NON_PIPE		(0x0 << 9)
#define SMC_BANK_TYPE2_PIPE		(0x1 << 9)

#define SMC_BANK_TYPE3_LATE_WRITE_D   	(0x0 << 8)	/* bit 8 */
#define SMC_BANK_TYPE3_LATE_WRITE_E   	(0x1 << 8)
 
 
#define SMC_BANK_TM_RBE_1		(0x01 << 20)	/* bit 20 */
 
#define SMC_BANK_TM_AST_0		(0x00 << 18)	/* bit 18-19 */
#define SMC_BANK_TM_AST_1		(0x01 << 18)
#define SMC_BANK_TM_AST_2		(0x02 << 18)
#define SMC_BANK_TM_AST_3		(0x03 << 18)

#define SMC_BANK_TM_CTW_1		(0x01 << 16)	/* bit 16-17 */
#define SMC_BANK_TM_CTW_2		(0x02 << 16)
#define SMC_BANK_TM_CTW_3		(0x03 << 16)

#define SMC_BANK_TM_AT1_1		(0x01 << 12)	/* bit 12-15 */
#define SMC_BANK_TM_AT1_2		(0x02 << 12)  
#define SMC_BANK_TM_AT1_3		(0x03 << 12)
#define SMC_BANK_TM_AT1_4		(0x04 << 12)
#define SMC_BANK_TM_AT1_5		(0x05 << 12)
#define SMC_BANK_TM_AT1_6		(0x06 << 12)
#define SMC_BANK_TM_AT1_7		(0x07 << 12)
#define SMC_BANK_TM_AT1_8		(0x08 << 12)
#define SMC_BANK_TM_AT1_9		(0x09 << 12)
#define SMC_BANK_TM_AT1_A		(0x0A << 12)
#define SMC_BANK_TM_AT1_B		(0x0B << 12)
#define SMC_BANK_TM_AT1_C		(0x0C << 12)
#define SMC_BANK_TM_AT1_D		(0x0D << 12)
#define SMC_BANK_TM_AT1_E		(0x0E << 12)
#define SMC_BANK_TM_AT1_F		(0x0F << 12)

#define SMC_BANK_TM_AT2_1		(0x01 << 8)	/* bit 9-8 */
#define SMC_BANK_TM_AT2_2		(0x02 << 8)
#define SMC_BANK_TM_AT2_3		(0x03 << 8)

#define SMC_BANK_TM_WTC_1		(0x01 << 6)	/* bit 7-6 */
#define SMC_BANK_TM_WTC_2		(0x02 << 6)
#define SMC_BANK_TM_WTC_3		(0x03 << 6)

#define SMC_BANK_TM_AHT_1		(0x01 << 4)	/* bit 5-4 */
#define SMC_BANK_TM_AHT_2		(0x02 << 4)
#define SMC_BANK_TM_AHT_3		(0x03 << 4)

#define SMC_BANK_TM_TRNA_1		(0x01)		/* bit 3-0 */
#define SMC_BANK_TM_TRNA_2		(0x02)
#define SMC_BANK_TM_TRNA_3		(0x03)
#define SMC_BANK_TM_TRNA_4		(0x04)
#define SMC_BANK_TM_TRNA_5		(0x05)
#define SMC_BANK_TM_TRNA_6		(0x06)
#define SMC_BANK_TM_TRNA_7		(0x07)
#define SMC_BANK_TM_TRNA_8		(0x08)
#define SMC_BANK_TM_TRNA_9		(0x09)
#define SMC_BANK_TM_TRNA_A		(0x0A)
#define SMC_BANK_TM_TRNA_B		(0x0B)
#define SMC_BANK_TM_TRNA_C		(0x0C)
#define SMC_BANK_TM_TRNA_D		(0x0D)
#define SMC_BANK_TM_TRNA_E		(0x0E)
#define SMC_BANK_TM_TRNA_F		(0x0F)



/*
 * macros declarations
 */
#define HAL_SMC_READ_MEMORY_BANK0_CONFIG(memory_bank0_config)			((memory_bank0_config) = (SMC_MEM_BANK0_CONFIG))


#define HAL_SMC_WRITE_MEMORY_BANK0_CONFIG(memory_bank0_config)			((SMC_MEM_BANK0_CONFIG) = (memory_bank0_config))


#define HAL_SMC_READ_MEMORY_BANK0_TIMING_PARAMETER(memory_bank0_timing)		((memory_bank0_timing) = (SMC_MEM_BANK0_TIMING_PARAM))


#define HAL_SMC_WRITE_MEMORY_BANK0_TIMING_PARAMETER(memory_bank0_timing)	((SMC_MEM_BANK0_TIMING_PARAM) = (memory_bank0_timing))


#define HAL_SMC_READ_MEMORY_BANK1_CONFIG(memory_bank1_config)			((memory_bank1_config) = (SMC_MEM_BANK1_CONFIG))


#define HAL_SMC_WRITE_MEMORY_BANK1_CONFIG(memory_bank1_config)			((SMC_MEM_BANK1_CONFIG) = (memory_bank1_config))


#define HAL_SMC_READ_MEMORY_BANK1_TIMING_PARAMETER(memory_bank1_timing)		((memory_bank1_timing) = (SMC_MEM_BANK1_TIMING_PARAM))


#define HAL_SMC_WRITE_MEMORY_BANK1_TIMING_PARAMETER(memory_bank1_timing)	((SMC_MEM_BANK1_TIMING_PARAM) = (memory_bank1_timing))


#define HAL_SMC_ENABLE_MEMORY_BANK0_WRITE_PROTECT()	((SMC_MEM_BANK0_CONFIG) |= (1 << 11))


#define HAL_SMC_DISABLE_MEMORY_BANK0_WRITE_PROTECT()	((SMC_MEM_BANK0_CONFIG) &= ~(1 << 11))


#define HAL_SMC_ENABLE_MEMORY_BANK1_WRITE_PROTECT()	((SMC_MEM_BANK1_CONFIG) |= (1 << 11))


#define HAL_SMC_DISABLE_MEMORY_BANK1_WRITE_PROTECT()	((SMC_MEM_BANK1_CONFIG) &= ~(1 << 11))


#endif  // end of #ifndef _STAR_SMC_H_