#ifndef _STAR_TIMER_H_
#define _STAR_TIMER_H_

/******************************************************************************
 * MODULE NAME:    star_timer.h
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           09 June 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    09 June 2003  -  MJLIU	- Initial Version v1.0
 *
 *
 * SOURCE:
 * ISSUES:
 * NOTES TO USERS:
 ******************************************************************************/

#include "sys_apb_memory_map.h"



#define TIMER_BASE_ADDR                 APB_DEVICES_MEM_MAP_ADDR(TIMER_BASE)

#define TIMER_TM1_COUNTER_REG_OFFSET    0x00
#define TIMER_TM1_LOAD_REG_OFFSET       0x04
#define TIMER_TM1_MATCH1_REG_OFFSET     0x08
#define TIMER_TM1_MATCH2_REG_OFFSET     0x0C

#define TIMER_TM2_COUNTER_REG_OFFSET    0x10
#define TIMER_TM2_LOAD_REG_OFFSET       0x14
#define TIMER_TM2_MATCH1_REG_OFFSET     0x18
#define TIMER_TM2_MATCH2_REG_OFFSET     0x1C

#define TIMER_TM_CR_REG_OFFSET          0x30
#define TIMER_TM_INTR_STATUS_REG_OFFSET 0x34
#define TIMER_TM_INTR_MASK_REG_OFFSET   0x38

#define TIMER_TM_REVISION_REG_OFFSET    0x3C


#define TIMER_MEM_MAP_ADDR(regname)     (TIMER_BASE_ADDR + TIMER_##regname##_OFFSET)

#define TIMER_MEM_MAP_VALUE(regname)    (*((u_int32 volatile *)TIMER_MEM_MAP_ADDR(regname)))


/*
 * define access macros
 */
#define TIMER1_COUNTER          TIMER_MEM_MAP_VALUE(TM1_COUNTER_REG)
#define TIMER1_LOAD             TIMER_MEM_MAP_VALUE(TM1_LOAD_REG)
#define TIMER1_MATCH1           TIMER_MEM_MAP_VALUE(TM1_MATCH1_REG)
#define TIMER1_MATCH2           TIMER_MEM_MAP_VALUE(TM1_MATCH2_REG)

#define TIMER2_COUNTER          TIMER_MEM_MAP_VALUE(TM2_COUNTER_REG)
#define TIMER2_LOAD             TIMER_MEM_MAP_VALUE(TM2_LOAD_REG)
#define TIMER2_MATCH1           TIMER_MEM_MAP_VALUE(TM2_MATCH1_REG)
#define TIMER2_MATCH2           TIMER_MEM_MAP_VALUE(TM2_MATCH2_REG)

#define TIMER_CR                TIMER_MEM_MAP_VALUE(TM_CR_REG)
#define TIMER_INTR_STATUS       TIMER_MEM_MAP_VALUE(TM_INTR_STATUS_REG)
#define TIMER_INTR_MASK         TIMER_MEM_MAP_VALUE(TM_INTR_MASK_REG)

#define TIMER_REVISION          TIMER_MEM_MAP_VALUE(TM_REVISION_REG)


/*
 * define constants macros
 */
#define TIMER1_ENABLE_BIT_INDEX             0
#define TIMER1_CLOCK_SOURCE_BIT_INDEX       1
#define TIMER1_OVERFLOW_ENABLE_BIT_INDEX    2

#define TIMER2_ENABLE_BIT_INDEX             3
#define TIMER2_CLOCK_SOURCE_BIT_INDEX       4
#define TIMER2_OVERFLOW_ENABLE_BIT_INDEX    5

#define TIMER1_UP_DOWN_COUNT_BIT_INDEX      9
#define TIMER2_UP_DOWN_COUNT_BIT_INDEX      10

#define TIMER1_MATCH1_INTR_BIT_INDEX        0
#define TIMER1_MATCH2_INTR_BIT_INDEX        1
#define TIMER1_OVERFLOW_INTR_BIT_INDEX      2

#define TIMER2_MATCH1_INTR_BIT_INDEX        3
#define TIMER2_MATCH2_INTR_BIT_INDEX        4
#define TIMER2_OVERFLOW_INTR_BIT_INDEX      5


#define TIMER_CLOCK_SOURCE_PCLK             0
#define TIMER_CLOCK_SOURCE_EXT_CLK          1


#define TIMER_OVERFLOW_MODE_DISABLE         0
#define TIMER_OVERFLOW_MODE_ENABLE          1


#define TIMER_COUNTER_MODE_UP               0
#define TIMER_COUNTER_MODE_DOWN             1


#define TIMER_1                             1
#define TIMER_2                             2


#define ACTION_READ                         0
#define ACTION_WRITE                        1


#define MATCH1_MASK_ENABLE                  (1 << 0)

#define MATCH2_MASK_ENABLE                  (1 << 1)

#define OVERFLOW_MASK_ENABLE                (1 << 2)



/*
 * macro declarations
 */
#define HAL_TIMER_ENABLE_TIMER1()			((TIMER_CR) |= (1 << TIMER1_ENABLE_BIT_INDEX))


#define HAL_TIMER_DISABLE_TIMER1()			((TIMER_CR) &= ~(1 << TIMER1_ENABLE_BIT_INDEX))


#define HAL_TIMER_ENABLE_TIMER2()			((TIMER_CR) |= (1 << TIMER2_ENABLE_BIT_INDEX))


#define HAL_TIMER_DISABLE_TIMER2()			((TIMER_CR) &= ~(1 << TIMER2_ENABLE_BIT_INDEX))

    
#define HAL_TIMER_ENABLE_TIMER1_OVERFLOW_MODE()		((TIMER_CR) |= (1 << TIMER1_OVERFLOW_ENABLE_BIT_INDEX))


#define HAL_TIMER_DISABLE_TIMER1_OVERFLOW_MODE()	((TIMER_CR) &= ~(1 << TIMER1_OVERFLOW_ENABLE_BIT_INDEX))


#define HAL_TIMER_ENABLE_TIMER2_OVERFLOW_MODE()		((TIMER_CR) |= (1 << TIMER2_OVERFLOW_ENABLE_BIT_INDEX))


#define HAL_TIMER_DISABLE_TIMER2_OVERFLOW_MODE()	((TIMER_CR) &= ~(1 << TIMER2_OVERFLOW_ENABLE_BIT_INDEX))

   
#define HAL_TIMER_SET_TIMER1_DOWNCOUNT()		((TIMER_CR) |= (1 << TIMER1_UP_DOWN_COUNT_BIT_INDEX))


#define HAL_TIMER_SET_TIMER1_UPCOUNT()			((TIMER_CR) &= ~(1 << TIMER1_UP_DOWN_COUNT_BIT_INDEX))

    
#define HAL_TIMER_SET_TIMER2_DOWNCOUNT()		((TIMER_CR) |= (1 << TIMER2_UP_DOWN_COUNT_BIT_INDEX))


#define HAL_TIMER_SET_TIMER2_UPCOUNT()			((TIMER_CR) &= ~(1 << TIMER2_UP_DOWN_COUNT_BIT_INDEX))


#define HAL_TIMER_READ_INTERRUPT_STATUS(interrupt_status)	((interrupt_status) = (TIMER_INTR_STATUS))


#define HAL_TIMER_WRITE_INTERRUPT_STATUS(interrupt_status)	((TIMER_INTR_STATUS) = (interrupt_status))


#define HAL_TIMER_READ_INTERRUPT_MASK(interrupt_mask)		((interrupt_mask) = (TIMER_INTR_MASK))


#define HAL_TIMER_WRITE_INTERRUPT_MASK(interrupt_mask)		((TIMER_INTR_MASK) = (interrupt_mask))


#define HAL_TIMER_MASK_TIMER1_TIMER2_ALL_INTERRUPTS()		((TIMER_INTR_MASK) = (0x3F))


#define HAL_TIMER_MASK_TIMER1_MATCH1_INTERRUPT()		((TIMER_INTR_MASK) |= (1 << TIMER1_MATCH1_INTR_BIT_INDEX))


#define HAL_TIMER_MASK_TIMER1_MATCH2_INTERRUPT()		((TIMER_INTR_MASK) |= (1 << TIMER1_MATCH2_INTR_BIT_INDEX))


#define HAL_TIMER_MASK_TIMER1_OVERFLOW_INTERRUPT()		((TIMER_INTR_MASK) |= (1 << TIMER1_OVERFLOW_INTR_BIT_INDEX))


#define HAL_TIMER_UNMASK_TIMER1_MATCH1_INTERRUPT()		((TIMER_INTR_MASK) &= ~(1 << TIMER1_MATCH1_INTR_BIT_INDEX))


#define HAL_TIMER_UNMASK_TIMER1_MATCH2_INTERRUPT()		((TIMER_INTR_MASK) &= ~(1 << TIMER1_MATCH2_INTR_BIT_INDEX))


#define HAL_TIMER_UNMASK_TIMER1_OVERFLOW_INTERRUPT()		((TIMER_INTR_MASK) &= ~(1 << TIMER1_OVERFLOW_INTR_BIT_INDEX))


#define HAL_TIMER_MASK_TIMER2_MATCH1_INTERRUPT()		((TIMER_INTR_MASK) |= (1 << TIMER2_MATCH1_INTR_BIT_INDEX))


#define HAL_TIMER_MASK_TIMER2_MATCH2_INTERRUPT()		((TIMER_INTR_MASK) |= (1 << TIMER2_MATCH2_INTR_BIT_INDEX))


#define HAL_TIMER_MASK_TIMER2_OVERFLOW_INTERRUPT()		((TIMER_INTR_MASK) |= (1 << TIMER2_OVERFLOW_INTR_BIT_INDEX))

    
#define HAL_TIMER_UNMASK_TIMER2_MATCH1_INTERRUPT()		((TIMER_INTR_MASK) &= ~(1 << TIMER2_MATCH1_INTR_BIT_INDEX))


#define HAL_TIMER_UNMASK_TIMER2_MATCH2_INTERRUPT()		((TIMER_INTR_MASK) &= ~(1 << TIMER2_MATCH2_INTR_BIT_INDEX))


#define HAL_TIMER_UNMASK_TIMER2_OVERFLOW_INTERRUPT()		((TIMER_INTR_MASK) &= ~(1 << TIMER2_OVERFLOW_INTR_BIT_INDEX))

#endif  // end of #ifndef _STAR_TIMER_H_