#ifndef _STAR_UART_H_
#define _STAR_UART_H_

/******************************************************************************
 * MODULE NAME:    star_uart.h
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           11 June 2003
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2003 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    11 June 2003  -  MJLIU	- Initial Version v1.0
 *
 *
 * SOURCE:
 * ISSUES:
 * NOTES TO USERS:
 ******************************************************************************/

#include "sys_apb_memory_map.h"

#define UART_BASE_ADDR           APB_DEVICES_MEM_MAP_ADDR(UART_BASE)

#define UART_RBR_REG_OFFSET      0x00
#define UART_THR_REG_OFFSET      0x00
#define UART_DLL_REG_OFFSET      0x00

#define UART_IER_REG_OFFSET      0x04
#define UART_DLM_REG_OFFSET      0x04

#define UART_IIR_REG_OFFSET      0x08
#define UART_FCR_REG_OFFSET      0x08
#define UART_PSR_REG_OFFSET      0x08

#define UART_LCR_REG_OFFSET      0x0C

#define UART_MCR_REG_OFFSET      0x10

#define UART_LSR_REG_OFFSET      0x14
#define UART_TST_REG_OFFSET      0x14

#define UART_MSR_REG_OFFSET      0x18

#define UART_SPR_REG_OFFSET      0x1C

#define UART_FEATURE_REG_OFFSET  0x68

#define UART_REVD1_REG_OFFSET    0x6C
#define UART_REVD2_REG_OFFSET    0x70
#define UART_REVD3_REG_OFFSET    0x74



#define UART_MEM_MAP_ADDR(regname)   (UART_BASE_ADDR + UART_##regname##_OFFSET)

#define UART_MEM_MAP_VALUE(regname)  (*((unsigned int volatile *)UART_MEM_MAP_ADDR(regname)))


/*
 * define access macros
 */
#define UART_RBR       UART_MEM_MAP_VALUE(RBR_REG)
#define UART_THR       UART_MEM_MAP_VALUE(THR_REG)
#define UART_IER       UART_MEM_MAP_VALUE(IER_REG)
#define UART_IIR       UART_MEM_MAP_VALUE(IIR_REG)
#define UART_FCR       UART_MEM_MAP_VALUE(FCR_REG)
#define UART_LCR       UART_MEM_MAP_VALUE(LCR_REG)
#define UART_MCR       UART_MEM_MAP_VALUE(MCR_REG)
#define UART_LSR       UART_MEM_MAP_VALUE(LSR_REG)
#define UART_TST       UART_MEM_MAP_VALUE(TST_REG)
#define UART_MSR       UART_MEM_MAP_VALUE(MSR_REG)

#define UART_DLL       UART_MEM_MAP_VALUE(DLL_REG)
#define UART_DLM       UART_MEM_MAP_VALUE(DLM_REG)
#define UART_PSR       UART_MEM_MAP_VALUE(PSR_REG)

#define UART_FEATURE   UART_MEM_MAP_VALUE(FEATURE_REG)

#define UART_REVD1     UART_MEM_MAP_VALUE(REVD1_REG)
#define UART_REVD2     UART_MEM_MAP_VALUE(REVD2_REG)
#define UART_REVD3     UART_MEM_MAP_VALUE(REVD3_REG)


/*
 * define constants macros
 */
#define UART_INPUT_CLOCK                  (48000000)

#define UART_FIFO_DEPTH                   16
 
#define RX_DATA_READY_INT                 (1 << 0)
#define THR_EMPTY_INT                     (1 << 1)
#define RX_LINE_STATUS_INT                (1 << 2)
#define MODEM_STATUS_INT                  (1 << 3)



#define NO_INT_PENDING_MASK               (0x1)
#define RX_LINE_STATUS_INT_MASK           (0x6)
#define RX_DATA_READY_INT_MASK            (0x4)
#define RX_DATA_TIMEOUT_INT_MASK          (0xC)
#define THR_EMPTY_INT_MASK                (0x2)
#define MODEM_STATUS_CHANGE_MASK          (0x0)


#define FIFO_ENABLE                       (1 << 0)
#define RX_FIFO_RESET                     (1 << 1)
#define TX_FIFO_RESET                     (1 << 2)
#define DMA_MODE                          (1 << 3)


#define RX_FIFO_TRIGGER_LEVEL_1           (0 << 6)
#define RX_FIFO_TRIGGER_LEVEL_4           (1 << 6)
#define RX_FIFO_TRIGGER_LEVEL_8           (2 << 6)
#define RX_FIFO_TRIGGER_LEVEL_14          (3 << 6)

#define WORD_LENGTH_5                     (0 << 0)
#define WORD_LENGTH_6                     (1 << 0)
#define WORD_LENGTH_7                     (2 << 0)
#define WORD_LENGTH_8                     (3 << 0)

#define STOP_BIT_1                        (0 << 2)
#define STOP_BIT_1_5                      (1 << 2)
#define STOP_BIT_2                        (1 << 2)

#define PARITY_CHECK_NONE                 (0 << 3)
#define PARITY_CHECK_EVEN                 (3 << 3)
#define PARITY_CHECK_ODD                  (1 << 3)
#define PARITY_CHECK_STICK_ONE            (5 << 3)
#define PARITY_CHECK_STICK_ZERO           (7 << 3)

#define SET_BREAK                         (1 << 6)

#define DLAB_ENABLE                       (1 << 7)

#define LOOPBACK_MODE                     (1 << 4)


#define DATA_READY                        (1 << 0)
#define OVERRUN_ERROR                     (1 << 1)
#define PARITY_ERROR                      (1 << 2)
#define FRAMING_ERROR                     (1 << 3)
#define BREAK_INTERRUPT                   (1 << 4)
#define THR_EMPTY                         (1 << 5)
#define TRANSMITTER_EMPTY                 (1 << 6)
#define FIFO_DATA_ERROR                   (1 << 7)

#define TEST_PARITY_ERROR                 (1 << 0)
#define TEST_FRAMING_ERROR                (1 << 1)
#define TEST_BAUD_GEN                     (1 << 2)
#define TEST_LOOPBACK_ENABLE              (1 << 3)


#define WORD_FIVE_BITS                    5
#define WORD_SIX_BITS                     6
#define WORD_SEVEN_BITS                   7
#define WORD_EIGHT_BITS                   8

#define NONE_PARITY                       1
#define EVEN_PARITY                       2
#define ODD_PARITY                        3
#define ONE_PARITY                        4
#define ZERO_PARITY                       5

#define ONE_STOP_BIT                      1
#define ONE_HALF_STOP_BIT                 2
#define TWO_STOP_BIT                      3

#define TX_RX_FIFO_DISABLE                0
#define TX_RX_FIFO_ENABLE                 1


/*
 * functions declarations
 */
void Uart_Configure_Baud_Rate(u_int32, u_int32);
void Uart_Configure_Line_Control(u_int32, u_int32, u_int32, u_int32);
void Uart_Configure_FIFO(u_int32, u_int32);


/*
 * macros declarations
 */
#define UART_READ_DATA(data)     ((data) = (UART_RBR) & 0xFF)
    
#define UART_WRITE_DATA(data)    ((UART_THR) = (data) & 0xFF)

#define UART_ENABLE_INTERRUPT_TYPE(interrupt_type)     ((UART_IER) |= (interrupt_type & 0xF))
    
#define UART_DISABLE_INTERRUPT_TYPE(interrupt_type)     ((UART_IER) &= ~(interrupt_type & 0xF))

#define UART_READ_INTERRUPT_IDENTIFICATION(uart_IIR)     ((uart_IIR) = (UART_IIR))

#define UART_CHECK_NO_INT_PENDING(uart_IIR)     (((uart_IIR) & 0xF) == (NO_INT_PENDING_MASK))
    
#define UART_CHECK_RX_LINE_STATUS_INT(uart_IIR)     (((uart_IIR) & 0xF) == (RX_LINE_STATUS_INT_MASK))
    
#define UART_CHECK_RX_DATA_READY_INT(uart_IIR)     (((uart_IIR) & 0xF) == (RX_DATA_READY_INT_MASK))
    
#define UART_CHECK_RX_DATA_TIMEOUT_INT(uart_IIR)     (((uart_IIR) & 0xF) == (RX_DATA_TIMEOUT_INT_MASK))
    
#define UART_CHECK_THR_EMPTY_INT(uart_IIR)     (((uart_IIR) & 0xF) == (THR_EMPTY_INT_MASK))

#define UART_CHECK_MODEM_STATUS_CHANGE_INT(uart_IIR)     (((uart_IIR) & 0xF) == (MODEM_STATUS_CHANGE_MASK))



#define UART_FIFO_ENABLE()     ((UART_FCR) |= (FIFO_ENABLE))

#define UART_FIFO_DISABLE()     ((UART_FCR) &= ~(FIFO_ENABLE))

#define UART_RESET_RX_FIFO()     ((UART_FCR) |= (RX_FIFO_RESET))
    
#define UART_RESET_TX_FIFO()     ((UART_FCR) |= (TX_FIFO_RESET))




#define UART_DLAB_ENABLE()     ((UART_LCR) |= (DLAB_ENABLE))
    
#define UART_DLAB_DISABLE()     ((UART_LCR) &= ~(DLAB_ENABLE))

#define UART_ENABLE_LOOPBACK_MODE()     ((UART_MCR) |= (LOOPBACK_MODE)) 

#define UART_DISABLE_LOOPBACK_MODE()     ((UART_MCR) &= ~(LOOPBACK_MODE))

#define UART_READ_LINE_STATUS(uart_LSR)     ((uart_LSR) = (UART_LSR))

#define UART_READ_MODEM_STATUS(uart_MSR)     ((uart_MSR) = (UART_MSR))

#define UART_WRITE_DLL(dll_value) {     UART_DLAB_ENABLE();    UART_DLL = (u_int32)dll_value;    UART_DLAB_DISABLE(); }

#define UART_WRITE_DLM(dlm_value) {     UART_DLAB_ENABLE();     UART_DLM = (u_int32)dlm_value;     UART_DLAB_DISABLE(); }

	
#define UART_WRITE_PSR(psr_value) {     UART_DLAB_ENABLE();     UART_PSR = ((u_int32)psr_value & 0x03);     UART_DLAB_DISABLE(); }



#define UART_READ_PSR(psr_value) {     UART_DLAB_ENABLE();     ((psr_value) = (UART_PSR) & 0x03);     UART_DLAB_DISABLE(); }

#define UART_CHECK_RX_DATA_READY()     (((UART_LSR) & DATA_READY) == (DATA_READY))
    
#define UART_CHECK_TX_FIFO_EMPTY()     (((UART_LSR) & THR_EMPTY) == (THR_EMPTY))

#define UART_CHECK_TRANSMITTER_EMPTY()     (((UART_LSR) & TRANSMITTER_EMPTY) == (TRANSMITTER_EMPTY))
    
#endif  // end of #ifndef _STAR_UART_H_
