#ifndef STR9100GSW_H
#define STR9100GSW_H



#include "porting.h"
#include "star_gsw.h"



/* ORION DMA Descriptor Define */

/* DMA Tx Desc Word #1   */
/* Data Pointer          */

/* DMA Tx Desc Word #2   */
#define DMA_TX_LEN_MSK          (0xffff)

#define DMA_TX_TCP_CGOL_EN      (1 << 16)
#define DMA_TX_UDP_CGOL_EN      (1 << 17)
#define DMA_TX_IP_CGOL_EN       (1 << 18)

#define DMA_TX_PORT_MAP         (19)
#define DMA_TX_GE_PORT_0        (1 << (DMA_TX_PORT_MAP + 0))
#define DMA_TX_GE_PORT_1        (1 << (DMA_TX_PORT_MAP + 1))
#define DMA_TX_CPU_PORT         (1 << (DMA_TX_PORT_MAP + 2))
#define DMA_TX_PORT_MAP_MSK     (0x7 << DMA_TX_PORT_MAP)

#define DMA_TX_PORT_MAP_FORCE   (1 << 22)

#define DMA_TX_PRIO             (23)
#define DMA_TX_PRIO_MSK         (0x7 << DMA_TX_PRIO)

#define DMA_TX_PRIO_FORCE       (1 << 26)
#define DMA_TX_INT_EN           (1 << 27)


/* DMA Tx Desc Word #3   */
#define DMA_TX_VLAN_ID_MSK      (0x7)

#define DMA_TX_INS_VLAN_ID      (1 << 3)

#define DMA_TX_SSN_ID           (4)
#define DMA_TX_SSN_ID_MSK       (0x7 << DMA_TX_SSN_ID)

#define DMA_TX_INS_SSN_ID       (1 << 7)

/* DMA Tx Desc Word #4   */
/* Wireless Tx Control   */


/* DMA Rx Desc Word #1   */
/* Data Pointer          */

/* DMA Rx Desc Word #2   */
#define DMA_RX_LEN_MSK          0xffff
#define DMA_RX_L4_CHK_FAIL      (1 << 16)
#define DMA_RX_IP_CHK_FAIL      (1 << 17)


#define DMA_RX_PROT             (18)
#define DMA_RX_PROT_MSK         (0x3 << DMA_RX_PROT)
#define DMA_RX_PROT_IP_ONLY     (0 << DMA_RX_PROT)
#define DMA_RX_PROT_IP_TCP      (1 << DMA_RX_PROT)
#define DMA_RX_PROT_IP_UDP      (2 << DMA_RX_PROT)
#define DMA_RX_PROT_OTHER       (3 << DMA_RX_PROT)


#define DMA_RX_HDR_RSN          (20)
#define DMA_RX_HDR_RSN_MSK      (0x3f << DMA_RX_HDR_RSN)


#define DMA_RX_SRC_PORT         (26)
#define DMA_RX_GE_PORT_0        (1 << (DMA_RX_SRC_PORT + 0))
#define DMA_RX_GE_PORT_1        (1 << (DMA_RX_SRC_PORT + 1))
#define DMA_RX_CPU_PORT         (1 << (DMA_RX_SRC_PORT + 2))
#define DMA_RX_SRC_PORT_MSK     (0x3 << DMA_RX_SRC_PORT)


/* TX, RX DMA reg */
#define DMA_LST_SEG             (1 << 28)
#define DMA_FST_SEG             (1 << 29)
#define DMA_END_OF_RING         (1 << 30)
#define DMA_CPU_OWN             (1 << 31)


/* DMA Rx Desc Word #3   */
/* Wireless Rx Status #1 */

/* DMA Rx Desc Word #4   */
/* Wireless Rx Status #2 */


/* specific PHY reg */
#define PHY_ADDR_0              0	// not used now
#define PHY_ADDR_1              1	// not used now

#define PHY_CTL_REG             0x0
#define PHY_AUTO_NEG_ADV_REG    0x4

#define PHY_AUTO_NEG_RSTRT      (1 << 9)

#define PHY_CMD_WAIT            0x10000000


typedef struct _orn_tx_desc_t_
{
	u32	buf_reg;
	u32	ctl_reg;
	u32	hdr_reg;
	u32	wlc_reg;
} orn_tx_desc;


typedef struct _orn_rx_desc_t_
{
	u32	buf_reg;
	u32	stat_reg;
	u32	wl_rxs0;
	u32	wl_rxs1;
} orn_rx_desc;


typedef struct _drv_pkt_desc_t_
{
	struct _drv_pkt_desc_t_	*next;
	
	u8	*buf;
	u32	len;
	u32	flag;
	u32	dst;
	u32	src;
} drv_pkt_desc;


typedef struct _drv_pkt_queue_t_
{
	drv_pkt_desc	*head;
	drv_pkt_desc	*tail;
} drv_pkt_queue;


typedef struct _orn_vlan_config_t_
{
	u16	gid;
	u16	vid;
	u32	port_map;
	u32	tag_port_map;
	u32	mac[2];
} orn_vlan_config;


typedef struct _orn_port_config_t_
{
	u32	pvid;
	u32	config;
	u32	state;
} orn_port_config;



#define NUM_RX_DESC     	10 //72

#define NUM_TX_DESC     	10 //72

#define NUM_FREE_DESC   	20 //144

#define PKT_BUFF_SIZE   	1536

#define PKT_MIN_LEN     	60

#define DEFAULT_VLAN_ID		1



typedef struct _orn_device_t_
{
	/*struct net_device_stats stats;*/

	orn_tx_desc	*hw_tx_desc;
	orn_rx_desc	*hw_rx_desc;
	
	drv_pkt_desc	**drv_tx_desc;
	drv_pkt_desc	**drv_rx_desc;
	drv_pkt_desc	*free_pkt_list;
	
	drv_pkt_queue	tx_pkt_queue;
			
	u32		int_msk;
	u32		ctl_flag;
	
	orn_port_config	port[3];
	orn_vlan_config	vlan[8];
	
	u32		num_rx_desc;
	u32		num_tx_desc;
	
	u32		rx_idx;
	
	u32		tx_idx_head;
	u32		tx_idx_tail;
	
	u32		rx_buf_full;
	
	u32		num_free_pkt;
	
	drv_pkt_desc	*pkt_pool;
	u8		*buf_pool;
	
	u32		mac_addr[2];
	
	u32		link_status;
	
	u32		cnt_tx_pkt;
	u32		cnt_rx_pkt;	
	u32		cnt_int;
	
} orn_device;


#define eth_outl(_addr, _value)        orn_outl(_addr, _value)
#define eth_outw(_addr, _value)        orn_outw(_addr, _value)
#define eth_outb(_addr, _value)        orn_outb(_addr, _value)

#define eth_inl(_addr)                 orn_inl(_addr)
#define eth_inw(_addr)                 orn_inw(_addr)
#define eth_inb(_addr)                 orn_inb(_addr)

#endif
