/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Date & Time support for RICOH RS5C372A
 */

 #undef	DEBUG	

#include <common.h>
#include <command.h>
#include <rtc.h>
#include <i2c.h>

#if defined(CONFIG_RTC_RS5C372A) && (CONFIG_COMMANDS & CFG_CMD_DATE)

/*
 * RTC register addresses
 */

#define RTC_SEC_REG		(0x00<<4)
#define RTC_MIN_REG		(0x01<<4)
#define RTC_HR_REG		(0x02<<4)
#define RTC_DAY_REG		(0x03<<4)
#define RTC_DATE_REG		(0x04<<4)
#define RTC_MON_REG		(0x05<<4)
#define RTC_YR_REG		(0x06<<4)
#define RTC_TIMING_REG		(0x07<<4)
#define RTC_ALARMA_MIN_REG	(0x08<<4)
#define RTC_ALARMA_HR_REG	(0x09<<4)
#define RTC_ALARMA_MIN_REG	(0x0A<<4)
#define RTC_ALARMB_MIN_REG	(0x0B<<4)
#define RTC_ALARMB_HR_REG	(0x0C<<4)
#define RTC_ALARMB_MIN_REG	(0x0D<<4)
#define RTC_CTL1_REG		(0x0E<<4)
#define RTC_CTL2_REG		(0x0F<<4)

#define RTC_READ_METHOD		0x04

#ifdef CFG_MATRIX500
static uchar i2c_reg_read(uchar devaddr, uchar reg);	
static void i2c_reg_write(uchar devaddr, uchar reg, uchar val);
#endif 

static uchar rtc_read  (uchar reg);
static void  rtc_write (uchar reg, uchar val);
static uchar bin2bcd   (unsigned int n);
static unsigned bcd2bin(uchar c);

/* ------------------------------------------------------------------------- */

void rtc_get (struct rtc_time *tmp)
{
	uchar sec, min, hour, mday, wday, mon_cent, year,ctl2;
	uchar buf[16];

//#ifdef CFG_MATRIX400
	ctl2	= rtc_read (RTC_CTL2_REG);
	debug("RTC reg[0x0f]:%02x\n", ctl2);	
	if(!(ctl2 & 0x20))
		rtc_write(RTC_CTL2_REG, ctl2|0x20);
	ctl2	= rtc_read (RTC_CTL2_REG);
	debug("RTC reg[0x0f]:%02x\n", ctl2);
	sec	= rtc_read (RTC_SEC_REG);
	min	= rtc_read (RTC_MIN_REG);
	hour	= rtc_read (RTC_HR_REG);
	mday	= rtc_read (RTC_DATE_REG);
	wday	= rtc_read (RTC_DAY_REG)-1;
	mon_cent= rtc_read (RTC_MON_REG);
	year	= rtc_read (RTC_YR_REG);
/*#else
	i2c_read(CFG_I2C_RTC_ADDR, 0xf0, 1, &buf, 1);
	debug("RTC reg[0x0f]:%02x\n", buf[0]);
	sec	= buf[0];
	min 	= buf[1];
	hour	= buf[2];
	wday	= buf[3]-1;
	mday	= buf[4];
	mon_cent= buf[5];
	year	= buf[6];
#endif*/
	
	debug ( "Get RTC year: %02x mon/cent: %02x mday: %02x wday: %02x "
		"hr: %02x min: %02x sec: %02x\n",
		year, mon_cent, mday, wday,
		hour, min, sec );
	/*debug ( "Alarms: wday: %02x  hour: %02x min: %02x\n",
		rtc_read (RTC_ALARMA_MIN_REG),
		rtc_read (RTC_ALARMA_HR_REG),
		rtc_read (RTC_ALARMA_MIN_REG) );
*/
	
	tmp->tm_sec  = bcd2bin (sec  & 0x7F);
	tmp->tm_min  = bcd2bin (min  & 0x7F);
	tmp->tm_hour = bcd2bin (hour & 0x3F);
	tmp->tm_mday = bcd2bin (mday & 0x3F);
	tmp->tm_mon  = bcd2bin (mon_cent & 0x1F);
	tmp->tm_year = bcd2bin (year) ;	
	if (tmp->tm_year >= 70)
		tmp->tm_year += 1900;
	else
		tmp->tm_year += 2000;
	tmp->tm_wday = bcd2bin (wday & 0x07);
	tmp->tm_yday = 0;
	tmp->tm_isdst= 0;

	debug ( "Get DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);
}

void rtc_set (struct rtc_time *tmp)
{
	
	debug ( "Set DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);

	if (tmp->tm_year < 1970 || tmp->tm_year > 2069)
			printf("WARNING: year should be between 1970 and 2069!\n");
	

	rtc_write (RTC_YR_REG, bin2bcd(tmp->tm_year % 100));
    	rtc_write (RTC_MON_REG, bin2bcd(tmp->tm_mon));
	rtc_write (RTC_DAY_REG, bin2bcd(tmp->tm_wday+1));
	rtc_write (RTC_DATE_REG, bin2bcd(tmp->tm_mday));
	rtc_write (RTC_HR_REG, bin2bcd(tmp->tm_hour));
	rtc_write (RTC_MIN_REG, bin2bcd(tmp->tm_min ));
	rtc_write (RTC_SEC_REG, bin2bcd(tmp->tm_sec ));

}

void rtc_reset (void)
{
	/* clear all control & status registers */
	rtc_write (RTC_CTL1_REG, 0x00);
	rtc_write (RTC_CTL2_REG, 0x00);
	/* reset all alarms */
	rtc_write (RTC_ALARMA_MIN_REG, 0x00);
	rtc_write (RTC_ALARMA_HR_REG, 0x00);
	rtc_write (RTC_ALARMA_MIN_REG, 0x00);
	rtc_write (RTC_ALARMB_MIN_REG, 0x00);
	rtc_write (RTC_ALARMB_HR_REG, 0x00);
	rtc_write (RTC_ALARMB_MIN_REG, 0x00);
}

/* ------------------------------------------------------------------------- */
#ifdef CFG_MATRIX500
static 
uchar i2c_reg_read(uchar devaddr, uchar reg){
	uchar buf;
	i2c_read(CFG_I2C_RTC_ADDR, reg, 1, &buf, 1);
	return buf;
}	
static 
void i2c_reg_write(uchar devaddr, uchar reg, uchar val){
	uchar buf=val;
	i2c_write(CFG_I2C_RTC_ADDR, reg, 1, &buf, 1);
}
#endif  

static uchar rtc_read (uchar reg)
{
#ifdef CFG_MATRIX400
	uchar buffer=0xff;
	i2c_read_ricoh2(CFG_I2C_RTC_ADDR, reg|RTC_READ_METHOD, 1, &buffer, 1);
	return buffer;
#else
	return (i2c_reg_read (CFG_I2C_RTC_ADDR, reg));
#endif
}

static void rtc_write (uchar reg, uchar val)
{

	i2c_reg_write (CFG_I2C_RTC_ADDR, reg, val);

}

static unsigned bcd2bin (uchar n)
{
	return ((((n >> 4) & 0x0F) * 10) + (n & 0x0F));
}

static unsigned char bin2bcd (unsigned int n)
{
	return (((n / 10) << 4) | (n % 10));
}

#endif	/* CONFIG_RTC_RS5C372A && CFG_CMD_DATE */
